﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SlimDX;
namespace LittleBigTable
{

    class Cubic
    {
        public short[] faces;
        public Face[] PositionFace;
        public Vertex[] vertices;
        public const int BUNNY_NUM_TRIANGLES = 12;
        public const int BUNNY_NUM_VERTICES = 36;
        public const int BUNNY_NUM_INDICES = BUNNY_NUM_TRIANGLES * 3;
        int[,] gIndicesBunny;
        double[] gVerticesBunny;

        public void CalculateTangentBinormal(TempVertexType vertex1, TempVertexType vertex2, TempVertexType vertex3, ref Vector3 tangent, ref Vector3 binormal)
        {
            float[] vector1 = new float[3];
            float[] vector2 = new float[3];
            float[] tuVector = new float[2];
            float[] tvVector = new float[2];
            float den;
            float length;


            // Calculate the two vectors for this face.
            vector1[0] = vertex2.x - vertex1.x;
            vector1[1] = vertex2.y - vertex1.y;
            vector1[2] = vertex2.z - vertex1.z;

            vector2[0] = vertex3.x - vertex1.x;
            vector2[1] = vertex3.y - vertex1.y;
            vector2[2] = vertex3.z - vertex1.z;
            // Calculate the tu and tv texture space vectors.



            tuVector[0] = 1.0f;
            tvVector[0] = 0.0f;

            tuVector[1] = 0.0f;
            tvVector[1] = 1.0f;

            // Calculate the denominator of the tangent/binormal equation.
            den = 1.0f / (tuVector[0] * tvVector[1] - tuVector[1] * tvVector[0]);

            // Calculate the cross products and multiply by the coefficient to get the tangent and binormal.
            tangent.X = (tvVector[1] * vector1[0] - tvVector[0] * vector2[0]) * den;
            tangent.Y = (tvVector[1] * vector1[1] - tvVector[0] * vector2[1]) * den;
            tangent.Z = (tvVector[1] * vector1[2] - tvVector[0] * vector2[2]) * den;

            binormal.X = (tuVector[0] * vector2[0] - tuVector[1] * vector1[0]) * den;
            binormal.Y = (tuVector[0] * vector2[1] - tuVector[1] * vector1[1]) * den;
            binormal.Z = (tuVector[0] * vector2[2] - tuVector[1] * vector1[2]) * den;

            //// Calculate the length of this normal.
            //length = tangent.Length();

            //// Normalize the normal and then store it
            //tangent.X = (float)(tangent.X / length);
            //tangent.Y = (float)(tangent.Y / length);
            //tangent.Z = (float)(tangent.Z / length);
            tangent.Normalize();

            // Calculate the length of this normal.
            //length = (float)Math.Sqrt((binormal.X * binormal.X) + (binormal.Y * binormal.Y) + (binormal.Z * binormal.Z));

            //// Normalize the normal and then store it
            //binormal.X = (float)(binormal.X / length);
            //binormal.Y = (float)(binormal.Y / length);
            //binormal.Z = (float)(binormal.Z / length);

            binormal.Normalize();
        }
        public void CalculateNormal(Vector3 tangent, Vector3 binormal, ref Vector3 normal)
        {
            float length;

            // Calculate the cross product of the tangent and binormal which will give the normal vector.
            normal.X = (tangent.Y * binormal.Z) - (tangent.Z * binormal.Y);
            normal.Y = (tangent.Z * binormal.X) - (tangent.X * binormal.Z);
            normal.Z = (tangent.X * binormal.Y) - (tangent.Y * binormal.X);

            // Calculate the length of the normal.
            length = (float)Math.Sqrt((normal.X * normal.X) + (normal.Y * normal.Y) + (normal.Z * normal.Z));

            // Normalize the normal.
            normal.X = normal.X / length;
            normal.Y = normal.Y / length;
            normal.Z = normal.Z / length;

            return;
        }
        public void CalculateNormalTangentBinormal()
        {
            int faceCount, i, index;
            TempVertexType vertex1, vertex2, vertex3;
            Vector3 tangent = new Vector3(0.0f, 0.0f, 0.0f);
            Vector3 binormal = new Vector3(0.0f, 0.0f, 0.0f);
            Vector3 normal = new Vector3(0.0f, 0.0f, 0.0f);


            vertex1 = new TempVertexType();
            vertex2 = new TempVertexType();
            vertex3 = new TempVertexType();


            // Calculate the number of faces in the model.
            faceCount = BUNNY_NUM_TRIANGLES;

            // Initialize the index to the model data.
            index = 0;

            // Go through all the faces and calculate the the tangent, binormal, and normal vectors.
            for (i = 0; i < faceCount; i++)
            {
                // Get the three vertices for this face from the model.

                vertex1.x = vertices[gIndicesBunny[i, 0]].pos.X;
                vertex1.y = vertices[gIndicesBunny[i, 0]].pos.Y;
                vertex1.z = vertices[gIndicesBunny[i, 0]].pos.Z;

                vertex2.x = vertices[gIndicesBunny[i, 1]].pos.X;
                vertex2.y = vertices[gIndicesBunny[i, 1]].pos.Y;
                vertex2.z = vertices[gIndicesBunny[i, 1]].pos.Z;


                vertex3.x = vertices[gIndicesBunny[i, 2]].pos.X;
                vertex3.y = vertices[gIndicesBunny[i, 2]].pos.Y;
                vertex3.z = vertices[gIndicesBunny[i, 2]].pos.Z;

                // Calculate the tangent and binormal of that face.
                CalculateTangentBinormal(vertex1, vertex2, vertex3, ref tangent, ref  binormal);

                // Calculate the new normal using the tangent and binormal.
                CalculateNormal(tangent, binormal, ref normal);

                // Store the normal, tangent, and binormal for this face back in the model structure.
                vertices[gIndicesBunny[i, 0]].normal = normal;
                vertices[gIndicesBunny[i, 0]].tangent = tangent;
                vertices[gIndicesBunny[i, 0]].binormal = binormal;

                vertices[gIndicesBunny[i, 1]].normal = normal;
                vertices[gIndicesBunny[i, 1]].tangent = tangent;
                vertices[gIndicesBunny[i, 1]].binormal = binormal;

                vertices[gIndicesBunny[i, 2]].normal = normal;
                vertices[gIndicesBunny[i, 2]].tangent = tangent;
                vertices[gIndicesBunny[i, 2]].binormal = binormal;

                PositionFace[i].normal = normal;
                //Set the 

                vertices[gIndicesBunny[i, 0]].tex = new Vector2(0.0f, 0.0f);
                vertices[gIndicesBunny[i, 1]].tex = new Vector2(0.5f, 0.0f);
                vertices[gIndicesBunny[i, 2]].tex = new Vector2(0.0f, 0.5f);

            }

        }
        public Cubic()
        {
            faces = new short[BUNNY_NUM_INDICES];
            PositionFace = new Face[BUNNY_NUM_TRIANGLES];
            for (int i = 0; i < BUNNY_NUM_TRIANGLES; i++)
            {
                PositionFace[i] = new Face();
            }
            vertices = new Vertex[BUNNY_NUM_VERTICES];
            for (int i = 0; i < BUNNY_NUM_VERTICES; i++)
            {
                vertices[i] = new Vertex();
            }


            gVerticesBunny = new double[BUNNY_NUM_VERTICES * 3]{
              -1.0f, 1.0f, -1.0f, 
           1.0f, 1.0f, -1.0f,
             -1.0f, -1.0f, -1.0f,
            -1.0f, -1.0f, -1.0f,
           1.0f, 1.0f, -1.0f,
            1.0f, -1.0f, -1.0f,
             1.0f, 1.0f, -1.0f,
           1.0f, 1.0f, 1.0f, 
            1.0f, -1.0f, -1.0f,
            1.0f, -1.0f, -1.0f,
           1.0f, 1.0f, 1.0f, 
             1.0f, -1.0f, 1.0f, 
             1.0f, 1.0f, 1.0f, 
            -1.0f, 1.0f, 1.0f, 
             1.0f, -1.0f, 1.0f, 
             1.0f, -1.0f, 1.0f, 
            -1.0f, 1.0f, 1.0f, 
             -1.0f, -1.0f, 1.0f,
            -1.0f, 1.0f, 1.0f, 
             -1.0f, 1.0f, -1.0f, 
             -1.0f, -1.0f, 1.0f, 
             -1.0f, -1.0f, 1.0f, 
            -1.0f, 1.0f, -1.0f, 
            -1.0f, -1.0f, -1.0f, 
            -1.0f, 1.0f, 1.0f,
           1.0f, 1.0f, 1.0f,
             -1.0f, 1.0f, -1.0f,
             -1.0f, 1.0f, -1.0f, 
           1.0f, 1.0f, 1.0f, 
           1.0f, 1.0f, -1.0f,
            -1.0f, -1.0f, -1.0f,
             1.0f, -1.0f, -1.0f, 
             -1.0f, -1.0f, 1.0f, 
             -1.0f, -1.0f, 1.0f,
             1.0f, -1.0f, -1.0f, 
             1.0f, -1.0f, 1.0f, 
            };





            gIndicesBunny = new int[12, 3];
            int idss = 0;
            for (int i = 0; i < 12; i++)
            {
                gIndicesBunny[i,0] = (int)idss;
                idss++;
                gIndicesBunny[i, 1] = (int)idss;
                idss++;
                gIndicesBunny[i, 2] = (int)idss;
                idss++;
            }

            int index = 0;
            int indexBunny = 0;
            int indexFace = 0;
            while (index < faces.Length)
            {

                PositionFace[indexBunny].v1 = new Vector3((float)gVerticesBunny[gIndicesBunny[indexBunny, 0] * 3],
                (float)gVerticesBunny[gIndicesBunny[indexBunny, 0] * 3 + 1],
                (float)gVerticesBunny[gIndicesBunny[indexBunny, 0] * 3 + 2]);

                PositionFace[indexBunny].v2 = new Vector3((float)gVerticesBunny[gIndicesBunny[indexBunny, 1] * 3],
               (float)gVerticesBunny[gIndicesBunny[indexBunny, 1] * 3 + 1],
               (float)gVerticesBunny[gIndicesBunny[indexBunny, 1] * 3 + 2]);

                PositionFace[indexBunny].v3 = new Vector3((float)gVerticesBunny[gIndicesBunny[indexBunny, 2] * 3],
               (float)gVerticesBunny[gIndicesBunny[indexBunny, 2] * 3 + 1],
               (float)gVerticesBunny[gIndicesBunny[indexBunny, 2] * 3 + 2]);

                PositionFace[indexBunny].position = (PositionFace[indexBunny].v1 + PositionFace[indexBunny].v2 + PositionFace[indexBunny].v3) / 3;

                Vector3 dv1 = PositionFace[indexBunny].v1 - PositionFace[indexBunny].position;
                Vector3 dv2 = PositionFace[indexBunny].v2 - PositionFace[indexBunny].position;
                Vector3 dv3 = PositionFace[indexBunny].v3 - PositionFace[indexBunny].position;

                PositionFace[indexBunny].radius = (float)Math.Max(dv1.Length(), Math.Max(dv2.Length(), dv3.Length()));





                faces[index] = (short)gIndicesBunny[indexBunny, 0];

                index++;
                faces[index] = (short)gIndicesBunny[indexBunny, 1];
                index++;
                faces[index] = (short)gIndicesBunny[indexBunny, 2];
                index++;
                indexBunny++;
            }
            //Compute the indices for 
            index = 0;

            for (int i = 0; i < vertices.Length; i++)
            {
                vertices[i].pos.X = (float)gVerticesBunny[index];
                index++;
                vertices[i].pos.Y = (float)gVerticesBunny[index];
                index++;
                vertices[i].pos.Z = (float)gVerticesBunny[index];
                index++;
                // vertices[i].tex = new SlimDX.Vector2(0.0f, 1.0f);
            }
            CalculateNormalTangentBinormal();

        }

    }

};
