﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SlimDX;
namespace LittleBigTable
{

    class Model1
    {
        public short[] faces;
        public Face[] PositionFace;
        public Vertex[] vertices;
        public const int BUNNY_NUM_TRIANGLES = 442;
        public const int BUNNY_NUM_VERTICES = 261;
        public const int BUNNY_NUM_INDICES = BUNNY_NUM_TRIANGLES * 3;
        int[,] gIndicesBunny;
        double[] gVerticesBunny;

        public void CalculateTangentBinormal(TempVertexType vertex1, TempVertexType vertex2, TempVertexType vertex3, ref Vector3 tangent, ref Vector3 binormal)
        {
            float[] vector1 = new float[3];
            float[] vector2 = new float[3];
            float[] tuVector = new float[2];
            float[] tvVector = new float[2];
            float den;
            float length;


            // Calculate the two vectors for this face.
            vector1[0] = vertex2.x - vertex1.x;
            vector1[1] = vertex2.y - vertex1.y;
            vector1[2] = vertex2.z - vertex1.z;

            vector2[0] = vertex3.x - vertex1.x;
            vector2[1] = vertex3.y - vertex1.y;
            vector2[2] = vertex3.z - vertex1.z;
            // Calculate the tu and tv texture space vectors.



            tuVector[0] = 1.0f;
            tvVector[0] = 0.0f;

            tuVector[1] = 0.0f;
            tvVector[1] = 1.0f;

            // Calculate the denominator of the tangent/binormal equation.
            den = 1.0f / (tuVector[0] * tvVector[1] - tuVector[1] * tvVector[0]);

            // Calculate the cross products and multiply by the coefficient to get the tangent and binormal.
            tangent.X = (tvVector[1] * vector1[0] - tvVector[0] * vector2[0]) * den;
            tangent.Y = (tvVector[1] * vector1[1] - tvVector[0] * vector2[1]) * den;
            tangent.Z = (tvVector[1] * vector1[2] - tvVector[0] * vector2[2]) * den;

            binormal.X = (tuVector[0] * vector2[0] - tuVector[1] * vector1[0]) * den;
            binormal.Y = (tuVector[0] * vector2[1] - tuVector[1] * vector1[1]) * den;
            binormal.Z = (tuVector[0] * vector2[2] - tuVector[1] * vector1[2]) * den;

            //// Calculate the length of this normal.
            //length = tangent.Length();

            //// Normalize the normal and then store it
            //tangent.X = (float)(tangent.X / length);
            //tangent.Y = (float)(tangent.Y / length);
            //tangent.Z = (float)(tangent.Z / length);
            tangent.Normalize();

            // Calculate the length of this normal.
            //length = (float)Math.Sqrt((binormal.X * binormal.X) + (binormal.Y * binormal.Y) + (binormal.Z * binormal.Z));

            //// Normalize the normal and then store it
            //binormal.X = (float)(binormal.X / length);
            //binormal.Y = (float)(binormal.Y / length);
            //binormal.Z = (float)(binormal.Z / length);

            binormal.Normalize();
        }
        public void CalculateNormal(Vector3 tangent, Vector3 binormal, ref Vector3 normal)
        {
            float length;

            // Calculate the cross product of the tangent and binormal which will give the normal vector.
            normal.X = (tangent.Y * binormal.Z) - (tangent.Z * binormal.Y);
            normal.Y = (tangent.Z * binormal.X) - (tangent.X * binormal.Z);
            normal.Z = (tangent.X * binormal.Y) - (tangent.Y * binormal.X);

            // Calculate the length of the normal.
            length = (float)Math.Sqrt((normal.X * normal.X) + (normal.Y * normal.Y) + (normal.Z * normal.Z));

            // Normalize the normal.
            normal.X = normal.X / length;
            normal.Y = normal.Y / length;
            normal.Z = normal.Z / length;

            return;
        }
        public void CalculateNormalTangentBinormal()
        {
            int faceCount, i, index;
            TempVertexType vertex1, vertex2, vertex3;
            Vector3 tangent = new Vector3(0.0f, 0.0f, 0.0f);
            Vector3 binormal = new Vector3(0.0f, 0.0f, 0.0f);
            Vector3 normal = new Vector3(0.0f, 0.0f, 0.0f);


            vertex1 = new TempVertexType();
            vertex2 = new TempVertexType();
            vertex3 = new TempVertexType();


            // Calculate the number of faces in the model.
            faceCount = BUNNY_NUM_TRIANGLES;

            // Initialize the index to the model data.
            index = 0;

            // Go through all the faces and calculate the the tangent, binormal, and normal vectors.
            for (i = 0; i < faceCount; i++)
            {
                // Get the three vertices for this face from the model.

                vertex1.x = vertices[gIndicesBunny[i, 0]].pos.X;
                vertex1.y = vertices[gIndicesBunny[i, 0]].pos.Y;
                vertex1.z = vertices[gIndicesBunny[i, 0]].pos.Z;

                vertex2.x = vertices[gIndicesBunny[i, 1]].pos.X;
                vertex2.y = vertices[gIndicesBunny[i, 1]].pos.Y;
                vertex2.z = vertices[gIndicesBunny[i, 1]].pos.Z;


                vertex3.x = vertices[gIndicesBunny[i, 2]].pos.X;
                vertex3.y = vertices[gIndicesBunny[i, 2]].pos.Y;
                vertex3.z = vertices[gIndicesBunny[i, 2]].pos.Z;

                // Calculate the tangent and binormal of that face.
                CalculateTangentBinormal(vertex1, vertex2, vertex3, ref tangent, ref  binormal);

                // Calculate the new normal using the tangent and binormal.
                CalculateNormal(tangent, binormal, ref normal);

                // Store the normal, tangent, and binormal for this face back in the model structure.
                vertices[gIndicesBunny[i, 0]].normal = normal;
                vertices[gIndicesBunny[i, 0]].tangent = tangent;
                vertices[gIndicesBunny[i, 0]].binormal = binormal;

                vertices[gIndicesBunny[i, 1]].normal = normal;
                vertices[gIndicesBunny[i, 1]].tangent = tangent;
                vertices[gIndicesBunny[i, 1]].binormal = binormal;

                vertices[gIndicesBunny[i, 2]].normal = normal;
                vertices[gIndicesBunny[i, 2]].tangent = tangent;
                vertices[gIndicesBunny[i, 2]].binormal = binormal;

                PositionFace[i].normal = normal;
                //Set the 

                vertices[gIndicesBunny[i, 0]].tex = new Vector2(0.0f, 0.0f);
                vertices[gIndicesBunny[i, 1]].tex = new Vector2(0.5f, 0.0f);
                vertices[gIndicesBunny[i, 2]].tex = new Vector2(0.0f, 0.5f);

            }

        }
        public Model1()
        {
            faces = new short[BUNNY_NUM_INDICES];
            PositionFace = new Face[BUNNY_NUM_TRIANGLES];
            for (int i = 0; i < BUNNY_NUM_TRIANGLES; i++)
            {
                PositionFace[i] = new Face();
            }
            vertices = new Vertex[BUNNY_NUM_VERTICES];
            for (int i = 0; i < BUNNY_NUM_VERTICES; i++)
            {
                vertices[i] = new Vertex();
            }


            gVerticesBunny = new double[BUNNY_NUM_VERTICES * 3]{
	(2.4377),	(-7.958),	(1.0658),	
	(2.5287),	(-7.671),	(1.4281),	
	(2.7176),	(-6.355),	(2.0493),	
	(2.7904),	(-5.8906),	(2.2045),	
	(2.8751),	(-5.1031),	(2.3632),	
	(2.8751),	(-4.4797),	(2.3484),	
	(2.8751),	(-4.167),	(2.3086),	
	(2.8751),	(-3.6633),	(2.2445),	
	(2.8751),	(-3.2081),	(2.2698),	
	(2.9116),	(-0.2423),	(2.4484),	
	(2.9116),	(0.0610999),	(2.4671),	
	(2.948),	(2.6452),	(2.6304),	
	(2.948),	(3.2033),	(2.6312),	
	(2.9237),	(3.8336),	(2.632),	
	(2.8751),	(4.4876),	(2.6329),	
	(2.8267),	(5.2019),	(2.5969),	
	(2.766),	(5.769),	(2.5471),	
	(1.8787),	(-8.1894),	(1.0973),	
	(1.8787),	(-7.8584),	(1.4401),	
	(1.8787),	(-6.3903),	(2.1589),	
	(1.8787),	(-5.9152),	(2.296),	
	(2.032),	(-5.0986),	(2.4733),	
	(2.5912),	(-4.4816),	(2.4586),	
	(2.6777),	(-4.2221),	(2.4117),	
	(2.7315),	(-3.6391),	(2.3403),	
	(2.7315),	(-3.2227),	(2.3603),	
	(2.7315),	(-0.2562),	(2.5265),	
	(2.7315),	(0.0627999),	(2.5473),	
	(2.5759),	(1.1928),	(2.6121),	
	(2.5759),	(2.6275),	(2.6951),	
	(2.5759),	(3.2067),	(2.6958),	
	(2.5759),	(3.8336),	(2.6965),	
	(2.5759),	(4.5073),	(2.6973),	
	(2.5759),	(5.2144),	(2.6808),	
	(2.5759),	(5.8069),	(2.6359),	
	(1.8787),	(-8.2419),	(0.147),	
	(2.4377),	(-7.9326),	(0.1323),	
	(2.6409),	(-7.7121),	(0.1382),	
	(3.167),	(-6.2556),	(0.1618),	
	(3.167),	(-6.2647),	(0.5764),	
	(3.167),	(-6.2501),	(0.9587),	
	(3.167),	(-6.0926),	(1.4596),	
	(3.167),	(-5.7842),	(1.8122),	
	(3.167),	(-5.1014),	(2.0468),	
	(3.167),	(-4.4792),	(1.9),	
	(3.167),	(-4.0116),	(1.443),	
	(3.167),	(-3.8644),	(0.957),	
	(3.167),	(-3.8496),	(0.4862),	
	(3.167),	(-3.8496),	(0.0733999),	
	(3.167),	(-3.559),	(0.0708),	
	(3.1276),	(-3.1691),	(0.0684999),	
	(3.0847),	(-2.836),	(0.0662001),	
	(3.1572),	(-0.586),	(0.049),	
	(3.1769),	(-0.0207),	(0.044),	
	(3.3847),	(2.6153),	(0.044),	
	(3.3847),	(2.5712),	(0.5464),	
	(3.3847),	(2.5712),	(0.9423),	
	(3.3847),	(2.7743),	(1.5322),	
	(3.3847),	(3.245),	(1.9294),	
	(3.3847),	(3.8376),	(2.0615),	
	(3.3847),	(4.5592),	(1.8995),	
	(3.3847),	(4.9421),	(1.4872),	
	(3.3847),	(5.1041),	(0.9275),	
	(3.3847),	(5.163),	(0.2354),	
	(3.2165),	(5.7531),	(0.3822),	
	(2.8751),	(6.1264),	(0.633),	
	(2.5759),	(6.4009),	(0.7633),	
	(1.7543),	(6.6449),	(0.8997),	
	(3.167),	(-4.1207),	(1.9086),	
	(3.1276),	(-3.5701),	(1.8014),	
	(3.0847),	(-3.1256),	(1.829),	
	(3.1474),	(-0.2209),	(2.0052),	
	(3.1966),	(0.1647),	(2.0261),	
	(3.3847),	(2.6517),	(2.0601),	
	(3.2165),	(5.4778),	(1.9689),	
	(2.8751),	(6.0531),	(2.0469),	
	(1.7785),	(-2.0843),	(3.2254),	
	(1.6574),	(-1.2013),	(3.3595),	
	(1.6251),	(-0.3333),	(3.385),	
	(1.6494),	(-0.0339),	(3.3657),	
	(1.7139),	(1.1999),	(3.289),	
	(1.7867),	(2.575),	(3.1512),	
	(1.8189),	(3.2069),	(3.0811),	
	(1.8593),	(3.8225),	(3.0127),	
	(1.9158),	(4.5501),	(2.9222),	
	(1.8512),	(5.2003),	(2.8224),	
	(1.7543),	(6.0177),	(2.6986),	
	(1.9542),	(-2.0843),	(3.0981),	
	(1.8125),	(-1.1677),	(3.2269),	
	(1.8043),	(-0.3268),	(3.2572),	
	(1.8303),	(-0.0314),	(3.2648),	
	(2.6409),	(-7.6912),	(1.0155),	
	(2.6409),	(-7.7173),	(0.5907),	
	(2.4377),	(-7.9182),	(0.5978),	
	(1.8787),	(-8.2145),	(0.5907),	
	(2.8751),	(6.1161),	(1.779),	
	(2.8751),	(6.136),	(1.126),	
	(1.8989),	(1.1989),	(3.1791),	
	(1.9759),	(2.5798),	(3.0494),	
	(3.2165),	(5.5389),	(1.6058),	
	(3.2165),	(5.6655),	(1.0003),	
	(1.9759),	(4.5368),	(2.8456),	
	(1.9759),	(3.8315),	(2.9259),	
	(2.5759),	(6.1242),	(2.0922),	
	(2.5759),	(6.1975),	(1.8123),	
	(2.5759),	(6.3175),	(1.1926),	
	(3.1769),	(0.00250005),	(0.5253),	
	(3.1572),	(-0.5681),	(0.5225),	
	(3.0847),	(-2.8657),	(0.511),	
	(3.1276),	(-3.2659),	(0.509),	
	(3.167),	(-3.5772),	(0.5074),	
	(3.1769),	(0.0026),	(0.975),	
	(3.1562),	(-0.4198),	(0.9755),	
	(3.0847),	(-2.9542),	(0.9785),	
	(3.1276),	(-3.3505),	(0.9789),	
	(3.167),	(-3.6142),	(0.9793),	
	(3.1966),	(0.1357),	(1.5568),	
	(3.1474),	(-0.2495),	(1.5512),	
	(3.0847),	(-3.0661),	(1.5103),	
	(3.1276),	(-3.4438),	(1.5048),	
	(3.167),	(-3.7626),	(1.5002),	
	(1.9759),	(3.2091),	(2.9875),	
	(-0.0153),	(-8.1013),	(1.4765),	
	(-0.0153),	(-8.4888),	(0.5758),	
	(-0.0153),	(-6.4028),	(2.2313),	
	(-0.0153),	(-5.059),	(2.5532),	
	(-0.0153),	(-4.4415),	(2.7526),	
	(-0.0153),	(-8.489),	(0.17),	
	(-0.0153),	(-2.1723),	(3.3411),	
	(-0.0153),	(-0.3484),	(3.5163),	
	(-0.0153),	(-0.0334001),	(3.5003),	
	(-0.0153),	(4.5706),	(2.9836),	
	(-0.0153),	(6.1398),	(2.7685),	
	(-0.0153),	(6.7858),	(0.9595),	
	(-2.6715),	(-7.6912),	(1.0155),	
	(-2.5592),	(-7.671),	(1.4281),	
	(-2.4682),	(-7.958),	(1.0658),	
	(-2.0065),	(4.5368),	(2.8456),	
	(-1.8817),	(5.2003),	(2.8224),	
	(-1.9463),	(4.5501),	(2.9222),	
	(-1.9092),	(-7.8584),	(1.4401),	
	(-1.9092),	(-8.1894),	(1.0973),	
	(-2.4682),	(-7.9182),	(0.5978),	
	(-2.6715),	(-7.7173),	(0.5907),	
	(-1.9092),	(-8.2145),	(0.5907),	
	(-1.9092),	(-6.3903),	(2.1589),	
	(-2.7482),	(-6.355),	(2.0493),	
	(-2.8209),	(-5.8906),	(2.2045),	
	(-1.9092),	(-5.9152),	(2.296),	
	(-2.9056),	(-5.1031),	(2.3632),	
	(-2.0626),	(-5.0986),	(2.4733),	
	(-2.9056),	(-4.4797),	(2.3484),	
	(-2.6217),	(-4.4816),	(2.4586),	
	(-2.9056),	(-4.167),	(2.3086),	
	(-2.7082),	(-4.2221),	(2.4117),	
	(-3.1975),	(-4.1207),	(1.9086),	
	(-3.1581),	(-3.5701),	(1.8014),	
	(-2.9056),	(-3.6633),	(2.2445),	
	(-2.762),	(-3.6391),	(2.3403),	
	(-3.1152),	(-3.1256),	(1.829),	
	(-2.9056),	(-3.2081),	(2.2698),	
	(-2.762),	(-3.2227),	(2.3603),	
	(-3.1779),	(-0.2209),	(2.0052),	
	(-3.2271),	(0.1647),	(2.0261),	
	(-2.9421),	(0.0610999),	(2.4671),	
	(-2.9421),	(-0.2423),	(2.4484),	
	(-2.762),	(0.0627999),	(2.5473),	
	(-2.762),	(-0.2562),	(2.5265),	
	(-2.9785),	(2.6452),	(2.6304),	
	(-2.6064),	(2.6275),	(2.6951),	
	(-2.6064),	(1.1928),	(2.6121),	
	(-2.9785),	(3.2033),	(2.6312),	
	(-2.6064),	(3.2067),	(2.6958),	
	(-2.9542),	(3.8336),	(2.632),	
	(-2.6064),	(3.8336),	(2.6965),	
	(-2.9056),	(4.4876),	(2.6329),	
	(-2.6064),	(4.5073),	(2.6973),	
	(-2.8572),	(5.2019),	(2.5969),	
	(-2.6064),	(5.2144),	(2.6808),	
	(-3.2471),	(5.4778),	(1.9689),	
	(-2.9056),	(6.0531),	(2.0469),	
	(-2.7965),	(5.769),	(2.5471),	
	(-2.6064),	(5.8069),	(2.6359),	
	(-2.6064),	(6.1242),	(2.0922),	
	(-0.0153),	(-8.4887),	(1.1319),	
	(-0.0153),	(-5.9202),	(2.3665),	
	(-1.809),	(-2.0843),	(3.2254),	
	(-1.8609),	(-0.0314),	(3.2648),	
	(-1.8349),	(-0.3268),	(3.2572),	
	(-1.9294),	(1.1989),	(3.1791),	
	(-2.0065),	(2.5798),	(3.0494),	
	(-2.0065),	(3.2091),	(2.9875),	
	(-2.0065),	(3.8315),	(2.9259),	
	(-1.7849),	(6.0177),	(2.6986),	
	(-1.7849),	(6.6449),	(0.8997),	
	(-1.9092),	(-8.2419),	(0.147),	
	(-1.688),	(-1.2013),	(3.3595),	
	(-0.0153),	(-1.2541),	(3.482),	
	(-1.6557),	(-0.3333),	(3.385),	
	(-1.6799),	(-0.0339),	(3.3657),	
	(-1.8898),	(3.8225),	(3.0127),	
	(-0.0153),	(3.8191),	(3.0772),	
	(-0.0153),	(5.2134),	(2.8936),	
	(-2.4682),	(-7.9326),	(0.1323),	
	(-2.6715),	(-7.7121),	(0.1382),	
	(-3.1975),	(-3.8644),	(0.957),	
	(-3.1975),	(-3.8496),	(0.4862),	
	(-3.1975),	(-3.5772),	(0.5074),	
	(-3.1975),	(-3.6142),	(0.9793),	
	(-3.1975),	(-3.559),	(0.0708),	
	(-3.1975),	(-3.8496),	(0.0733999),	
	(-3.1581),	(-3.1691),	(0.0684999),	
	(-3.1581),	(-3.2659),	(0.509),	
	(-3.1152),	(-2.836),	(0.0662001),	
	(-3.1152),	(-2.8657),	(0.511),	
	(-3.1878),	(-0.586),	(0.049),	
	(-3.2074),	(-0.0207),	(0.044),	
	(-3.2074),	(0.00250005),	(0.5253),	
	(-3.1878),	(-0.5681),	(0.5225),	
	(-3.4152),	(2.5712),	(0.9423),	
	(-3.4152),	(2.7743),	(1.5322),	
	(-3.2271),	(0.1357),	(1.5568),	
	(-3.2074),	(0.0026),	(0.975),	
	(-3.4152),	(4.9421),	(1.4872),	
	(-3.4152),	(5.1041),	(0.9276),	
	(-3.2471),	(5.6655),	(1.0003),	
	(-3.2471),	(5.5389),	(1.6058),	
	(-3.4152),	(4.5592),	(1.8995),	
	(-3.2471),	(5.7531),	(0.3822),	
	(-3.4152),	(5.163),	(0.2354),	
	(-2.9056),	(6.1264),	(0.633),	
	(-2.9056),	(6.136),	(1.126),	
	(-2.6064),	(6.4009),	(0.7633),	
	(-2.6064),	(6.3175),	(1.1926),	
	(-3.1975),	(-3.7626),	(1.5002),	
	(-3.1581),	(-3.4438),	(1.5048),	
	(-3.1152),	(-3.0661),	(1.5103),	
	(-3.1779),	(-0.2495),	(1.5512),	
	(-2.9056),	(6.1161),	(1.779),	
	(-2.6064),	(6.1975),	(1.8123),	
	(-1.9847),	(-2.0843),	(3.0981),	
	(-1.843),	(-1.1677),	(3.2269),	
	(-1.7444),	(1.1999),	(3.289),	
	(-1.8172),	(2.575),	(3.1512),	
	(-1.8495),	(3.2069),	(3.0811),	
	(-3.1867),	(-0.4198),	(0.9755),	
	(-3.1152),	(-2.9542),	(0.9785),	
	(-3.1581),	(-3.3505),	(0.9789),	
	(-3.1975),	(-6.2556),	(0.1618),	
	(-3.1975),	(-6.2647),	(0.5764),	
	(-3.1975),	(-5.7842),	(1.8122),	
	(-3.1975),	(-5.1014),	(2.0468),	
	(-3.1975),	(-4.4792),	(1.9),	
	(-3.1975),	(-4.0116),	(1.443),	
	(-3.4152),	(2.6517),	(2.0601),	
	(-3.4152),	(3.245),	(1.9294),	
	(-3.4152),	(3.8376),	(2.0615),	
	(-3.4152),	(2.5712),	(0.5464),	
	(-3.4152),	(2.6153),	(0.044),	
	(-3.1975),	(-6.2501),	(0.9587),	
	(-3.1975),	(-6.0926),	(1.4596)

};


            gIndicesBunny = new int[,]{
	{91,1,0 }, {101,85,84 }, {0,1,18 }, {18,17,0 }, {93,92,91 },
	{91,0,93 }, {17,94,93 }, {93,0,17 }, {124,122,18 }, {18,19,124 },
	{2,3,20 }, {20,19,2 }, {3,4,21 }, {21,20,3 }, {4,5,22 }, {22,21,4 },
	{5,6,23 }, {23,22,5 }, {68,69,7 }, {7,6,68 }, {6,7,24 }, {24,23,6 },
	{69,70,8 }, {8,7,69 }, {7,8,25 }, {25,24,7 }, {71,72,10 }, {10,9,71 },
	{9,10,27 }, {27,26,9 }, {11,29,28 }, {11,28,27 }, {10,11,27 },
	{11,12,30 }, {30,29,11 }, {12,13,31 }, {31,30,12 }, {13,14,32 },
	{32,31,13 }, {14,15,33 }, {33,32,14 }, {74,75,16 }, {16,15,74 },
	{15,16,34 }, {34,33,15 }, {16,75,103 }, {103,34,16 }, {17,18,122 },
	{122,184,17 }, {20,185,124 }, {124,19,20 }, {20,21,125 }, {125,185,20 },
	{21,22,126 }, {126,125,21 }, {76,128,126 }, {126,22,76 }, {26,27,90 },
	{90,89,26 }, {27,28,97 }, {97,90,27 }, {28,29,98 }, {98,97,28 },
	{29,30,121 }, {121,98,29 }, {30,31,102 }, {102,121,30 }, {31,32,101 },
	{101,102,31 }, {32,33,85 }, {85,101,32 }, {33,34,86 }, {86,85,33 },
	{133,132,86 }, {86,67,133 }, {94,123,127 }, {127,35,94 }, {77,197,128 },
	{128,76,77 }, {78,129,197 }, {197,77,78 }, {78,79,130 }, {130,129,78 },
	{83,84,131 }, {131,201,83 }, {85,202,131 }, {131,84,85 }, {86,132,202 },
	{202,85,86 }, {35,36,93 }, {93,94,35 }, {36,37,92 }, {92,93,36 },
	{46,47,110 }, {110,115,46 }, {110,47,48 }, {48,49,110 }, {49,50,109 },
	{109,110,49 }, {50,51,108 }, {108,109,50 }, {52,53,106 }, {106,107,52 },
	{56,57,116 }, {116,111,56 }, {61,62,100 }, {61,100,99 }, {61,99,74 },
	{61,74,60 }, {100,62,63 }, {63,64,100 }, {64,65,96 }, {96,100,64 },
	{65,66,105 }, {105,96,65 }, {120,119,69 }, {69,68,120 }, {119,118,70 },
	{70,69,119 }, {26,25,8 }, {8,9,26 }, {117,116,72 }, {72,71,117 },
	{99,95,75 }, {75,74,99 }, {75,95,104 }, {104,103,75 }, {87,88,77 },
	{77,76,87 }, {88,89,78 }, {78,77,88 }, {89,90,79 }, {79,78,89 },
	{90,97,80 }, {80,79,90 }, {97,98,81 }, {81,80,97 }, {98,121,82 },
	{82,81,98 }, {121,102,83 }, {83,82,121 }, {102,101,84 }, {84,83,102 },
	{95,96,105 }, {105,104,95 }, {99,100,96 }, {96,95,99 }, {111,112,107 },
	{107,106,111 }, {113,114,109 }, {109,108,113 }, {114,115,110 },
	{110,109,114 }, {116,117,112 }, {112,111,116 }, {112,117,118 },
	{118,113,112 }, {118,119,114 }, {114,113,118 }, {119,120,115 },
	{115,114,119 }, {39,92,37 }, {37,38,39 }, {3,2,42 }, {3,42,43 },
	{4,3,43 }, {120,68,44 }, {44,45,120 }, {120,45,46 }, {46,115,120 },
	{68,6,5 }, {5,4,43 }, {5,43,44 }, {68,5,44 }, {11,73,58 }, {58,12,11 },
	{73,57,58 }, {12,58,59 }, {59,13,12 }, {14,13,59 }, {59,60,14 },
	{15,14,60 }, {60,74,15 }, {70,71,9 }, {9,8,70 }, {118,117,71 },
	{71,70,118 }, {113,108,107 }, {107,112,113 }, {51,52,107 }, {107,108,51 },
	{72,116,57 }, {57,73,72 }, {106,55,56 }, {56,111,106 }, {53,54,55 },
	{55,106,53 }, {72,73,11 }, {11,10,72 }, {86,34,103 }, {86,103,104 },
	{105,66,67 }, {104,105,67 }, {86,104,67 }, {22,23,87 }, {87,76,22 },
	{18,1,2 }, {2,19,18 }, {1,91,40 }, {1,40,41 }, {1,41,42 }, {1,42,2 },
	{91,92,39 }, {39,40,91 }, {94,17,184 }, {184,123,94 }, {130,79,80 },
	{80,81,82 }, {130,80,82 }, {130,82,83 }, {130,83,201 }, {134,136,135 },
	{137,139,138 }, {136,141,140 }, {140,135,136 }, {142,136,134 },
	{134,143,142 }, {141,136,142 }, {142,144,141 }, {124,145,140 },
	{140,122,124 }, {146,145,148 }, {148,147,146 }, {147,148,150 },
	{150,149,147 }, {149,150,152 }, {152,151,149 }, {151,152,154 },
	{154,153,151 }, {155,153,157 }, {157,156,155 }, {153,154,158 },
	{158,157,153 }, {156,157,160 }, {160,159,156 }, {157,158,161 },
	{161,160,157 }, {162,165,164 }, {164,163,162 }, {165,167,166 },
	{166,164,165 }, {170,169,168 }, {166,170,168 }, {164,166,168 },
	{168,169,172 }, {172,171,168 }, {171,172,174 }, {174,173,171 },
	{173,174,176 }, {176,175,173 }, {175,176,178 }, {178,177,175 },
	{179,177,181 }, {181,180,179 }, {177,178,182 }, {182,181,177 },
	{181,182,183 }, {183,180,181 }, {141,184,122 }, {122,140,141 },
	{148,145,124 }, {124,185,148 }, {148,185,125 }, {125,150,148 },
	{150,125,126 }, {126,152,150 }, {186,152,126 }, {126,128,186 },
	{167,188,187 }, {187,166,167 }, {166,187,189 }, {189,170,166 },
	{170,189,190 }, {190,169,170 }, {169,190,191 }, {191,172,169 },
	{172,191,192 }, {192,174,172 }, {174,192,137 }, {137,176,174 },
	{176,137,138 }, {138,178,176 }, {178,138,193 }, {193,182,178 },
	{133,194,193 }, {193,132,133 }, {144,195,127 }, {127,123,144 },
	{196,186,128 }, {128,197,196 }, {198,196,197 }, {197,129,198 },
	{198,129,130 }, {130,199,198 }, {200,201,131 }, {131,139,200 },
	{138,139,131 }, {131,202,138 }, {193,138,202 }, {202,132,193 },
	{195,144,142 }, {142,203,195 }, {203,142,143 }, {143,204,203 },
	{205,208,207 }, {207,206,205 }, {207,209,210 }, {210,206,207 },
	{209,207,212 }, {212,211,209 }, {211,212,214 }, {214,213,211 },
	{215,218,217 }, {217,216,215 }, {219,222,221 }, {221,220,219 },
	{223,227,179 }, {223,179,226 }, {223,226,225 }, {223,225,224 },
	{225,228,229 }, {229,224,225 }, {228,225,231 }, {231,230,228 },
	{230,231,233 }, {233,232,230 }, {234,155,156 }, {156,235,234 },
	{235,156,159 }, {159,236,235 }, {167,165,160 }, {160,161,167 },
	{237,162,163 }, {163,221,237 }, {226,179,180 }, {180,238,226 },
	{180,183,239 }, {239,238,180 }, {240,186,196 }, {196,241,240 },
	{241,196,198 }, {198,188,241 }, {188,198,199 }, {199,187,188 },
	{187,199,242 }, {242,189,187 }, {189,242,243 }, {243,190,189 },
	{190,243,244 }, {244,191,190 }, {191,244,200 }, {200,192,191 },
	{192,200,139 }, {139,137,192 }, {238,239,233 }, {233,231,238 },
	{226,238,231 }, {231,225,226 }, {222,217,218 }, {218,245,222 },
	{246,214,212 }, {212,247,246 }, {247,212,207 }, {207,208,247 },
	{221,222,245 }, {245,237,221 }, {245,246,236 }, {236,237,245 },
	{236,246,247 }, {247,235,236 }, {235,247,208 }, {208,234,235 },
	{249,248,204 }, {204,143,249 }, {250,146,147 }, {251,250,147 },
	{149,251,147 }, {234,253,252 }, {252,155,234 }, {234,208,205 },
	{205,253,234 }, {251,149,151 }, {252,251,151 }, {155,252,151 },
	{155,151,153 }, {168,171,255 }, {255,254,168 }, {254,255,220 },
	{171,173,256 }, {256,255,171 }, {175,227,256 }, {256,173,175 },
	{177,179,227 }, {227,175,177 }, {159,160,165 }, {165,162,159 },
	{236,159,162 }, {162,237,236 }, {246,245,218 }, {218,214,246 },
	{213,214,218 }, {218,215,213 }, {163,254,220 }, {220,221,163 },
	{217,222,219 }, {219,257,217 }, {216,217,257 }, {257,258,216 },
	{163,164,168 }, {168,254,163 }, {194,232,233 }, {194,233,239 },
	{193,194,239 }, {193,239,183 }, {193,183,182 }, {152,186,240 },
	{240,154,152 }, {140,145,146 }, {146,135,140 }, {135,146,250 },
	{135,250,260 }, {135,260,259 }, {135,259,134 }, {134,259,249 },
	{249,143,134 }, {144,123,184 }, {184,141,144 }, {130,201,200 },
	{130,200,244 }, {244,243,242 }, {130,244,242 }, {130,242,199 }
};

            int index = 0;
            int indexBunny = 0;
            int indexFace = 0;
            while (index < faces.Length)
            {

                PositionFace[indexBunny].v1 = new Vector3((float)gVerticesBunny[gIndicesBunny[indexBunny, 0] * 3],
                (float)gVerticesBunny[gIndicesBunny[indexBunny, 0] * 3 + 1],
                (float)gVerticesBunny[gIndicesBunny[indexBunny, 0] * 3 + 2]);

                PositionFace[indexBunny].v2 = new Vector3((float)gVerticesBunny[gIndicesBunny[indexBunny, 1] * 3],
               (float)gVerticesBunny[gIndicesBunny[indexBunny, 1] * 3 + 1],
               (float)gVerticesBunny[gIndicesBunny[indexBunny, 1] * 3 + 2]);

                PositionFace[indexBunny].v3 = new Vector3((float)gVerticesBunny[gIndicesBunny[indexBunny, 2] * 3],
               (float)gVerticesBunny[gIndicesBunny[indexBunny, 2] * 3 + 1],
               (float)gVerticesBunny[gIndicesBunny[indexBunny, 2] * 3 + 2]);

                PositionFace[indexBunny].position = (PositionFace[indexBunny].v1 + PositionFace[indexBunny].v2 + PositionFace[indexBunny].v3) / 3;

                Vector3 dv1 = PositionFace[indexBunny].v1 - PositionFace[indexBunny].position;
                Vector3 dv2 = PositionFace[indexBunny].v2 - PositionFace[indexBunny].position;
                Vector3 dv3 = PositionFace[indexBunny].v3 - PositionFace[indexBunny].position;

                PositionFace[indexBunny].radius = (float)Math.Max(dv1.Length(), Math.Max(dv2.Length(), dv3.Length()));





                faces[index] = (short)gIndicesBunny[indexBunny, 0];

                index++;
                faces[index] = (short)gIndicesBunny[indexBunny, 1];
                index++;
                faces[index] = (short)gIndicesBunny[indexBunny, 2];
                index++;
                indexBunny++;
            }
            //Compute the indices for 
            index = 0;

            for (int i = 0; i < vertices.Length; i++)
            {
                vertices[i].pos.X = (float)gVerticesBunny[index];
                index++;
                vertices[i].pos.Y = (float)gVerticesBunny[index];
                index++;
                vertices[i].pos.Z = (float)gVerticesBunny[index];
                index++;
                // vertices[i].tex = new SlimDX.Vector2(0.0f, 1.0f);
            }
            CalculateNormalTangentBinormal();

        }

    }

};
