﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SlimDX;
using SlimDX.D3DCompiler;
using SlimDX.Direct3D11;
using SlimDX.DXGI;
using Device = SlimDX.Direct3D11.Device;
using Resource = SlimDX.Direct3D11.Resource;

namespace LittleBigTable
{
    public enum TextEffect { NONE, FADE_IN, FADE_OUT, FADE_IN_OUT };
    public class Text
    {
        public string Context;
        SlimDX.Direct2D.SolidColorBrush brushColor;
        protected SlimDX.DirectWrite.TextFormat textFormat;
        string Style;
        int transformedX;
        int transformedY; 
        int Size;
        public Vector3 worldPosition;
        public Vector3 Color;
        public float Alpha{get;set;}
        TextEffect Effect;
        bool UpAlpha;
        float FadeRate;
        public bool IsCreated { get; set; }
        public Vector2 ScreenPosition { get; set; }
        public bool IsWorldDraw { get; set;}
        
        public Text(string context,string style, int size, Vector3 color)
        {
            IsWorldDraw = true;
            IsCreated = true;
            worldPosition = new Vector3();
            Context = context;
            Style = style;
            Size = size;
            Color = color;
            //Alpha = color.W;
            Effect = TextEffect.NONE;
            FadeRate = 0.0f;
        }
        public void TriggerEffect(TextEffect effect,float fadeRate)
        {
            Effect = effect;
            FadeRate = fadeRate;
        }
        public  void ApplyEffect()
        {
            switch (Effect)
            {
                //Now we apply effect of text
                //No effect
                    
                case TextEffect.NONE:
                {
                    UpAlpha = true;
                    break;
                }
                case TextEffect.FADE_IN:
                {
                    if (Alpha < 0.9f)
                    {
                        Alpha += FadeRate;
                    }
                    else
                    {
                        Effect = TextEffect.NONE;
                    }
                    break;
                }

                case TextEffect.FADE_OUT:
                    if (Alpha > 0.0f)
                    {
                        Alpha -= FadeRate;
                    }
                    else
                    {
                        Effect = TextEffect.NONE;
                    }
                    break;
                case TextEffect.FADE_IN_OUT:
                    if (UpAlpha)
                    {
                        if (Alpha < 0.9f)
                        {
                            Alpha += FadeRate;
                        }
                        else
                        {
                            UpAlpha = false;
                        }
                    }
                    if (!UpAlpha)
                    {
                        if (Alpha > 0.0f)
                        {
                            Alpha -= FadeRate;
                        }
                        else
                        {
                            UpAlpha = true;
                            Effect = TextEffect.NONE;
                        }
                    }
                    break;
            }
        }
        public void SetPosition(Vector3 position)
        {
            worldPosition = position;
        }
        public void InitializeText(SlimDX.Direct2D.RenderTarget dwRenderTarget,SlimDX.DirectWrite.Factory dwFactory)
        {
            //Initialize textFormat
            //dwFactory = new SlimDX.DirectWrite.Factory(
            //    SlimDX.DirectWrite.FactoryType.Isolated
            //);


            textFormat = new SlimDX.DirectWrite.TextFormat(
               dwFactory,
               Style,
               SlimDX.DirectWrite.FontWeight.Normal,
               SlimDX.DirectWrite.FontStyle.Normal,
               SlimDX.DirectWrite.FontStretch.Normal,
               Size,
               "en-US"
           );
           textFormat.TextAlignment = SlimDX.DirectWrite.TextAlignment.Center;
           textFormat.ParagraphAlignment = SlimDX.DirectWrite.ParagraphAlignment.Center;
            //Initialize brushColor
           brushColor = new SlimDX.Direct2D.SolidColorBrush(
               dwRenderTarget,
               new Color4(1, Color.X, Color.Y, Color.Z));

           
        }
        public void DrawScreenPosition(Vector2 screenPosition)
        {
            ScreenPosition=screenPosition;
            IsWorldDraw=false;
        }
        public void drawText( Camera camera, SlimDX.Direct2D.RenderTarget dwRenderTarget)
        {

            Matrix worldViewProj = camera.GetViewMatrix() * camera.GetProjectionMatrix();
            float x = worldPosition.X * worldViewProj.get_Columns(0).X + worldPosition.Y * worldViewProj.get_Columns(0).Y + worldPosition.Z * worldViewProj.get_Columns(0).Z + worldViewProj.get_Columns(0).W;
            float y = worldPosition.X * worldViewProj.get_Columns(1).X + worldPosition.Y * worldViewProj.get_Columns(1).Y + worldPosition.Z * worldViewProj.get_Columns(1).Z + worldViewProj.get_Columns(1).W;
            float z = worldPosition.X * worldViewProj.get_Columns(2).X + worldPosition.Y * worldViewProj.get_Columns(2).Y + worldPosition.Z * worldViewProj.get_Columns(2).Z + worldViewProj.get_Columns(2).W;
            float w = worldPosition.X * worldViewProj.get_Columns(3).X + worldPosition.Y * worldViewProj.get_Columns(3).Y + worldPosition.Z * worldViewProj.get_Columns(3).Z + worldViewProj.get_Columns(3).W;
            Vector4 PosOnScreen = new Vector4(x, y, z, w);
            Vector2 screenCoord;
            screenCoord.X = PosOnScreen.X / PosOnScreen.W;
            screenCoord.Y = -PosOnScreen.Y / PosOnScreen.W;
            screenCoord.X = screenCoord.X / 2.0f + 0.5f;
            screenCoord.Y = screenCoord.Y / 2.0f + 0.5f;

            Vector2 wndPos;
            wndPos.X = screenCoord.X * camera.GetWidth();
            wndPos.Y = screenCoord.Y * camera.GetHeight();
            brushColor.Opacity = Alpha;
            //int transformedX=(int)((int)wndPos.X * 1000 / camera.GetWidth() - 500);
            //int transformedY = (int)((int)wndPos.Y * 1000 / camera.GetWidth() - 500);
           float realX= transformedX * 0.5f + (wndPos.X - camera.GetWidth() / 2) * 0.5f;
           float realY = transformedY * 0.5f + (wndPos.Y - camera.GetHeight() / 2) * 0.5f;

           transformedX = (int)realX;
           transformedY = (int)realY;

           if (IsWorldDraw)
           {
               dwRenderTarget.DrawText(Context, textFormat, new System.Drawing.Rectangle(transformedX,
               transformedY, camera.GetWidth(), camera.GetHeight()), brushColor);
           }
           else
           {
               dwRenderTarget.DrawText(Context, textFormat, new System.Drawing.Rectangle((int)ScreenPosition.X,
               (int)ScreenPosition.Y, camera.GetWidth(), camera.GetHeight()), brushColor);
           }
        }
        
    }
}
