﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SlimDX;
using SlimDX.D3DCompiler;
using SlimDX.Direct3D11;

//using SlimDX.DXGI;

namespace LittleBigTable
{
    //float4 pos : POSITION;
    //float2 tex : TEXCOORD0;
    //float3 normal : NORMAL;
    //float3 tangent : TANGENT;
    //float3 binormal : BINORMAL;
    public struct TempVertexType
    {
        public float x, y, z;
        public float tu, tv;
        public float nx, ny, nz;
    };
    public class Constraint
    {

        public virtual void SolveConstraint(ref Vector3 accelerate, Vector3 velocity, Vector3 position)
        {

        }
    }
    public class AirFriction: Constraint
    {
        public float Cofficient;
        public AirFriction(float cofficient)
        {
            Cofficient=cofficient;
        }
        public override void SolveConstraint(ref Vector3 accelerate, Vector3 velocity, Vector3 position)
        {
            accelerate = accelerate - Cofficient * velocity;
        }
    }
    public class Spring : Constraint
    {
        public float Stiffness;
        public Vector3 SpringPosition;
        public float SpringDamping;
        public float Mass;
        public float RestLength;
        public Spring(float mass,float stiffness,float springDamping,float restLength ,Vector3 springPosition)
        {
            Mass = mass;
            SpringPosition = springPosition;
            RestLength = restLength;
            Stiffness = stiffness;
            SpringDamping = springDamping;
            
        }
        public override void SolveConstraint(ref Vector3 accelerate, Vector3 velocity, Vector3 position)
        {
            Vector3 v = (SpringPosition-position)/ (position - SpringPosition).Length();
            Vector3 l = ((position - SpringPosition).Length() - RestLength )* v;
            accelerate = accelerate + Stiffness * l / Mass;
        }
    }
    
    public class Vertex
    {
        public Vector3 pos;
        public Vector2 tex;
        public Vector3 normal;
        public Vector3 tangent;
        public Vector3 binormal;
        public Vertex(float[] ptn)
        {
            pos.X = ptn[0]; pos.Y = ptn[1]; pos.Z = ptn[2];
            tex.X = ptn[3]; tex.Y = ptn[4];
            normal.X = ptn[5]; normal.Y = ptn[6]; normal.Z = ptn[7];
        }
        public Vertex()
        {
            
        }
        

    }
    public class Face
    {
        public Vector3 v1;
        public Vector3 v2;
        public Vector3 v3;
        public Vector3 center;
        public Vector3 position;
        public Vector3 normal;
        public float radius;
        
    }
    public class ZD3DObject
    {
        Matrix worldMatrix;
        public List<Constraint> ConstraintList;
        public Vector3 Position;
        public Vector3 Position1;
        public Vector3 Position2;
        public Vector3 Position3;
        public Vector3 Position4;
        public Vector3 Scale;
        public int indexSplit;
        public Vector3 Velocity;
        public Vector3 Velocity1;
        public Vector3 Velocity2;
        public Vector3 Velocity3;
        public Vector3 Velocity4;
        public Vector3 AngleAxis;
        public Vector3 Accelerate;
        public Vector3 Accelerate1;
        public Vector3 Accelerate2;
        public Vector3 Accelerate3;
        public Vector3 Accelerate4;
        public Vertex[] WorldVertics;
        public List<int> indices;
        public float Radius;
        int vertexSize;
        SlimDX.Direct3D11.Buffer vertexBuffer;
        public float[] vertices ;
        //public Constraint
        Quaternion quaternion;
        public float Angle;
        public Vector3 AngularVelocity;
        public float Mass;
        protected ShaderResourceView colorTextureRV, normalTextureRV;
        bool isStatic;
        public Face[] PositionFace;
        public Face[] WorldPositionFace;
        public short [] Faces;
        public Vertex[] Vertices;
        public Vector3[] PurePosition;
        public Vector3[] PurePositionWorldPosition;
        public Vector3[] verticesPosition;
        public bool IsRigidBody;
        int vertexCount;
        public ZD3DObject(Device device, Shader shader, string colorName,string normalName,TextureManager textureMgr)
        {
            Position = new Vector3(0.0f, 0.0f, 0.0f);
            Velocity = new Vector3(0.0f, 0.0f, 0.0f);
            Accelerate = new Vector3(0.0f, 0.0f, 0.0f);
            Scale = new Vector3(10.0f, 10.0f, 10.0f);
           // AngleAxis = new Vector3(0.0f, 0.r0f, 1.0f);
            //Angle = 0.0f;
            SetTextures(device, shader, colorName, normalName, textureMgr);
            IsRigidBody = false;
            Mass = 500;
            ConstraintList = new List<Constraint>();
            
            //createCube(device);
        }
        //public void CollisionDetection(ZD3DObject obj, List<ZD3DObject> list)
        //{
        //    for (int i = 0; i < indices.Count; i++)
        //    {
        //        list[indices[i]]
        //    }
        //}
        public bool CollisionDetectionObj2Obj(ZD3DObject obj)
        {
            // Relative velocity
            Vector3 dv = Velocity - obj.Velocity;// obj2->prVelocity - obj1->prVelocity;
            // Relative position
            Vector3 dp = Position - obj.Position;
            //Minimal distance squared
            float r = Radius + obj.Radius;
           
            //dP^2-r^2
            float pp = dp.X * dp.X + dp.Y * dp.Y + dp.Z * dp.Z - r * r;
            //(1)Check if the spheres are already intersecting
            if (pp < 0) return true;
            else
            {
                return false;
            }

            ////dP*dV
            //float pv = dp.X * dv.X + dp.Y * dv.Y + dp.Z * dv.Z;
            ////(2)Check if the spheres are moving away from each other
            //if (pv >= 0) return false;

            ////dV^2
            //float vv = dv.X * dv.X + dv.Y * dv.Y + dv.Z * dv.Z;
            ////(3)Check if the spheres can intersect within 1 frame
            //if ((pv + vv) <= 0 && (vv + 2 * pv + pp) >= 0) return false;

            ////Discriminant/4
            //float D = pv * pv - pp * vv;
            //return (D > 0);
        }
        public bool NarrowphaseCollisionDetection(ZD3DObject obj)
        {
           
            for (int i = 0; i < PositionFace.Length; i++)
            {
              
                for (int j = 0; j < obj.PositionFace.Length; j++)
                {

                     
                    // Face fworld1 =PositionFace[i];
                    //fworld1.position=Multiply( fworld1.position,worldMatrix);
                    // Face fworld2 = obj.PositionFace[j];
                    // fworld2.position = Multiply(fworld2.position, obj.worldMatrix);

                    //if ((fworld1.position - fworld2.position).Length() - (fworld1.radius + fworld2.radius) < 0.0f)
                    //{
                    //    return true;
                    //}
                   
                    //if (Face2Face(PositionFace[i], obj.PositionFace[j], obj))
                    //{
                    //    return true;
                    //}
                }
            }

            return false;
        }
        public Vector3 SupportPoint(Vector3 Direction)
        {
            Vector3 result;
            float maxdot = float.MinValue;
            result = Vector3.Zero;
            UpdateVertices();
            int i = 0;
            int maxIndex = 0; ;
            for(;i<WorldVertics.Length;i++)
            {
                Vector3 v = WorldVertics[i].pos - Position;
                float tempdot = Vector3.Dot(Direction, v);

                if (tempdot > maxdot)
                {
                    maxdot = tempdot;
                    maxIndex = i;
                    result = WorldVertics[i].pos;
                }
            }
            indexSplit = maxIndex;
            return result;
        }
        
        bool getLinePlaneIntersectionPoint(Vector3 vLineStart, Vector3  vLineEnd,
                                    Vector3 vPointInPlane, Vector3 vPlaneNormal,
                                    ref Vector3 vIntersection)
        {
            Vector3 vDirection;
            Vector3 L1;
            float fLineLength;
            float fDistanceFromPlane;
            float fPercentage;

            vDirection.X = vLineEnd.X - vLineStart.X;
            vDirection.Y = vLineEnd.Y - vLineStart.Y;
            vDirection.Z = vLineEnd.Z - vLineStart.Z;

            fLineLength = Vector3.Dot(vDirection,vPlaneNormal);
                
               

            // Check the line's length allowing for some tolerance for floating point
            // rounding errors. If it's 0 or really close to 0, the line is parallel to
            // the plane and can not intersect it.
            if (Math.Abs(fLineLength) < 0.001f)
                return false;

            L1.X = vPointInPlane.X - vLineStart.X;
            L1.Y = vPointInPlane.Y - vLineStart.Y;
            L1.Z = vPointInPlane.Z - vLineStart.Z;

            fDistanceFromPlane = Vector3.Dot(L1, vPlaneNormal);

            // How far from Linestart , intersection is as a percentage of 0 to 1
            fPercentage = fDistanceFromPlane / fLineLength;

            if (fPercentage < 0.0f || // The plane is behind the start of the line
                fPercentage > 1.0f)  // The line segment does not reach the plane
                return false;

            // Add the percentage of the line to line start
            vIntersection.X = vLineStart.X + vDirection.X * fPercentage;
            vIntersection.Y = vLineStart.Y + vDirection.Y * fPercentage;
            vIntersection.Z = vLineStart.Z + vDirection.Z * fPercentage;

            return true;
        }
        bool isPointInsideTriangle( Vector3 vIntersectionPoint, Face tri )
        {
	        Vector3 [] vVectors=new Vector3[3];
	        float fTotalAngle = 0.0f; // As radians

	        //
	        // Create and normalize three vectors that radiate out from the
	        // intersection point towards the triangle's three vertices.
	        //

            vVectors[0] = vIntersectionPoint - tri.v1;
	        vVectors[0].Normalize();

	        vVectors[1] = vIntersectionPoint - tri.v2;
            vVectors[1].Normalize();

	        vVectors[2] = vIntersectionPoint - tri.v3;
            vVectors[2].Normalize();

	        //
	        // We then sum together the angles that exist between each of the vectors.
	        //
	        // Here's how:
	        //
	        // 1. Use dotProduct() to get cosine of the angle between the two vectors.
	        // 2. Use acos() to convert cosine back into an angle.
	        // 3. Add angle to fTotalAngle to keep track of running sum.
	        //

	        fTotalAngle  = (float)Math.Cos( Vector3.Dot( vVectors[0], vVectors[1] ) );
            fTotalAngle += (float)Math.Cos(Vector3.Dot(vVectors[1], vVectors[2]));
            fTotalAngle += (float)Math.Cos(Vector3.Dot(vVectors[2], vVectors[0]));

	        //
	        // If we are able to sum together all three anlges and get 360.0, the
	        // intersection point is inside the triangle.
	        //
	        // We can check this by taking away 6.28 radians (360 degrees) away from
	        // fTotalAngle and if we're left with 0 (allowing for some tolerance) the
	        // intersection point is definitely inside the triangle.
	        //

	        if( Math.Abs( fTotalAngle - 6.28f ) < 0.01f )
		        return true;

	        return false;
        }

        public bool Face2Face(Face f1,Face f2,ZD3DObject obj)
        {

            //Convert to worldPosition.
            Face fworld1 = new Face();
            fworld1.v1 = Multiply(f1.v1, worldMatrix);
            fworld1.v2 = Multiply(f1.v2, worldMatrix);
            fworld1.v3 = Multiply(f1.v3, worldMatrix);

            
            Face fworld2 = new Face();
            fworld2.v1 = Multiply(f2.v1, obj.worldMatrix);
            fworld2.v2 = Multiply(f2.v2, obj.worldMatrix);
            fworld2.v3 = Multiply(f2.v3, obj.worldMatrix);



            

            bool bIntersect = false;
            Vector3 vPoint=new Vector3(0.0f,0.0f,0.0f);

            //
            // Create a normal for 'tri1'
            //

            Vector3 vEdgeVec1 = fworld1.v2 - fworld1.v1;
            Vector3 vEdgeVec2 = fworld1.v3 - fworld1.v1;

            Vector3 vNormal1 = Vector3.Cross(vEdgeVec1, vEdgeVec2);// Crso.crossProduct(vEdgeVec1, vEdgeVec2);
            //tri1.vNormal.normalize(); // Some people feel compelled to normalize this, but it's not really necessary.

            //
            // Check the first line segment of triangle #2 against triangle #1
            //
            // The first line segment is defined by vertices v0 and v1.
            //

            bIntersect = getLinePlaneIntersectionPoint(fworld2.v1,      // Line start
                                                        fworld2.v2,      // Line end
                                                        fworld1.v1,      // A point in the plane
                                                        vNormal1, // The plane's normal
                                                        ref vPoint);     // Holds the intersection point, if the function returns true
            if (bIntersect == true)
            {
                //
                // The line segment intersects the plane, but does it actually go 
                // through the triangle?
                //

                if (isPointInsideTriangle(vPoint, fworld1) == true)
                    return true;
            }

            //
            // Check the second line segment of triangle #2 against triangle #1
            //
            // The second line segment is defined by vertices v1 and v2.
            //

            bIntersect = getLinePlaneIntersectionPoint(fworld1.v2,      // Line start
                                                        fworld2.v3,      // Line end
                                                        fworld1.v1,      // A point in the plane
                                                        vNormal1, // The plane's normal
                                                        ref vPoint);     // Holds the intersection point, if the function returns true

            if (bIntersect == true)
            {
                //
                // The line segment intersects the plane, but does it actually go 
                // through the triangle?
                //

                if (isPointInsideTriangle(vPoint, fworld1) == true)
                    return true;
            }

            //
            // Check the third line segment of triangle #2 against triangle #1
            //
            // The third line segment is defined by vertices v2 and v0.
            //

            bIntersect = getLinePlaneIntersectionPoint(fworld2.v3,      // Line start
                                                        fworld2.v1,      // Line end
                                                        fworld1.v1,      // A point in the plane
                                                        vNormal1, // The plane's normal
                                                        ref vPoint);     // Holds the intersection point, if the function returns true

            if (bIntersect == true)
            {
                //
                // The line segment intersects the plane, but does it actually go 
                // through the triangle?
                //

                if (isPointInsideTriangle(vPoint, fworld1) == true)
                    return true;
            }

            return false;
        }
        public Vector3 Multiply(Vector3 v, Matrix rotation)
        {
            float x = v.X * rotation.get_Columns(0).X + v.Y * rotation.get_Columns(0).Y + v.Z * rotation.get_Columns(0).Z+rotation.get_Columns(0).W;
            float y = v.X * rotation.get_Columns(1).X + v.Y * rotation.get_Columns(1).Y + v.Z * rotation.get_Columns(1).Z + rotation.get_Columns(1).W;
            float z = v.X * rotation.get_Columns(2).X + v.Y * rotation.get_Columns(2).Y + v.Z * rotation.get_Columns(2).Z + rotation.get_Columns(2).W;

            return new Vector3(x, y, z);
        }
        public void SetRotation(float angle, Vector3 angleAxis)
        {
            angleAxis = angle * angleAxis;
            quaternion = Quaternion.RotationAxis(angleAxis, angleAxis.Length());
        }
        public void MultiplyRotation(float angle, Vector3 angleAxis)
        {
            quaternion = Quaternion.Multiply(quaternion, Quaternion.RotationAxis(angleAxis, angle));
        }
        public void EularSover(float dt)
        {
           // AngularVelocity *= 0.99f;
            Position = Position + dt * Velocity;
            Velocity = Velocity + dt * SolveAccelerate(Position,Velocity , dt);
            AngleAxis = AngleAxis + AngularVelocity*dt;

            Vector3 Dir = AngleAxis;
            Dir.Normalize();
            while (AngleAxis.Length() > Math.PI * 2)
            {
                AngleAxis = (float)(AngleAxis.Length() - 2 * Math.PI) * Dir;
            }
            
            quaternion = Quaternion.RotationAxis(Dir, AngleAxis.Length());

            Accelerate = SolveAccelerate(Position, Velocity, dt);
        }
        public void AddConstraint(Constraint constraint)
        {
            ConstraintList.Add(constraint);
        }
        public void MidPointSolver(float dt)
        {
            Position1 = Position;

            Velocity1 = Velocity;

            Accelerate1 = SolveAccelerate(Position1, Velocity1, 0.0f);

            Position2 = Position + 0.5f * Velocity1 * dt;

            Velocity2 = Velocity + 0.5f * Accelerate1 * dt;

            Accelerate2 = SolveAccelerate(Position2, Velocity2, dt / 2.0f);


            Position = Position + Velocity2 * dt;

            Velocity = Velocity + Accelerate2* dt;

            Accelerate = Accelerate2;
        }
        public void RungeKutta4(float dt)
        {
             
             Position1=Position;
          
             Velocity1=Velocity;
           
             Accelerate1=SolveAccelerate(Position1,Velocity1,0.0f);
          
             Position2=Position+0.5f*Velocity1*dt;
           
             Velocity2=Velocity+0.5f*Accelerate1*dt;
            
             Accelerate2=SolveAccelerate(Position2,Velocity2,dt/2.0f);

             
             Position3=Position+0.5f*Velocity2*dt;
       
             Velocity3=Velocity+0.5f*Accelerate2*dt;
       
             Accelerate3=SolveAccelerate(Position3,Velocity3,dt/2.0f);

       
             Position4=Position+0.5f*Velocity3*dt;
     
             Velocity4=Velocity+Accelerate3*dt;
      
             Accelerate4=SolveAccelerate(Position4,Velocity4,dt);
             
             Position = Position + (dt/6.0f)*(Velocity1 + 2*Velocity2 + 2*Velocity3 + Velocity4);
             Velocity = Velocity + (dt/6.0f)*(Accelerate1 + 2*Accelerate2 + 2*Accelerate3 + Accelerate4);

             Accelerate = (Accelerate1 + 2 * Accelerate2 + 2 * Accelerate3 + Accelerate4) / 6.0f;
        }
        
        public Vector3 SolveAccelerate(Vector3 position,Vector3 velocity , float dt)
        {
            Vector3 accelerate=new Vector3(0.0f,0.0f,0.0f);
            accelerate = accelerate + new Vector3(0.0f, 0.0f, -10.0f);//Gravity
            //float RealPlane = 30.0f - Scale.Z;
            //if (position.Z > RealPlane)//Collision withe Ground
            //{
            //    if (velocity.Z > 0.0f)
            //    {
            //        accelerate = accelerate + new Vector3(0.0f, 0.0f, 1000.0f * (RealPlane - Position.Z));//
            //    }
            //    else
            //    {
            //        accelerate = accelerate + new Vector3(0.0f, 0.0f, 100.0f * (RealPlane - Position.Z));
            //    }

            //    accelerate = accelerate - 8.0f * new Vector3(velocity.X / velocity.Length(), velocity.Y / velocity.Length(), 0);//Friction
            //}
            //accelerate = accelerate - 0.1f * velocity;

            //foreach (Constraint a in ConstraintList)
            //{
            //    a.SolveConstraint(ref accelerate, velocity, position);
            //}

            return accelerate;
        }

        public bool SetTextures(Device device, Shader shader, string colorTextureFileName, string normalTextureFileName, TextureManager textureMgr)
        {


            if (colorTextureFileName != null)
            {

                colorTextureRV = new ShaderResourceView(device, textureMgr.MatchTextures(device, colorTextureFileName));
                // shader.GetEffect().GetVariableByName("colorTexture").AsResource().SetResource(colorTextureRV);
            }
            if (normalTextureFileName != null)
            {

                normalTextureRV = new ShaderResourceView(device, textureMgr.MatchTextures(device, normalTextureFileName));
                // shader.GetEffect().GetVariableByName("normalTexture").AsResource().SetResource(normalTextureRV);
            }

            return true;
        }
        public void Render(Shader shader, Device device, Lighting lighting, Camera camera)
        {
            //Update Vertices to be rendered
            UpdateVertices(device);
           
            Quaternion scaleR = new Quaternion(0.0f, 0.0f, 0.0f, 0.0f);
           worldMatrix = Matrix.Transformation(new Vector3(0.0f, 0.0f, 0.0f), scaleR,
          Scale, new Vector3(0.0f, 0.0f, 0.0f), quaternion, Position);

           // LoadModel(device);

            shader.GetEffect().GetVariableByName("colorTexture").AsResource().SetResource(colorTextureRV);
            shader.GetEffect().GetVariableByName("normalTexture").AsResource().SetResource(normalTextureRV);
            
            shader.Set3DModel(device, Position, Scale, quaternion, 1.0f);
            shader.Render(device, lighting, camera, vertexCount);

            UpdateFaces();
            UpdateVertices();

           

        }
      
        public void createCube(Device device)
        {
            vertexCount = 36;
            Vertices = new Vertex[36];
            PositionFace = new Face[12];
            float[] ptn1 = { -1.0f, 1.0f, -1.0f, 0.0f, 0.0f, 0.0f, 0.0f, -1.0f }; Vertices[0] = new Vertex(ptn1);
            float[] ptn2 = { 1.0f, 1.0f, -1.0f, 1.0f, 0.0f, 0.0f, 0.0f, -1.0f }; Vertices[1] = new Vertex(ptn2);
            float[] ptn3 = { -1.0f, -1.0f, -1.0f, 0.0f, 1.0f, 0.0f, 0.0f, -1.0f }; Vertices[2] = new Vertex(ptn3);
            float[] ptn4 = { -1.0f, -1.0f, -1.0f, 0.0f, 1.0f, 0.0f, 0.0f, -1.0f }; Vertices[3] = new Vertex(ptn4);
            float[] ptn5 = { 1.0f, 1.0f, -1.0f, 1.0f, 0.0f, 0.0f, 0.0f, -1.0f }; Vertices[4] = new Vertex(ptn5);
            float[] ptn6 = { 1.0f, -1.0f, -1.0f, 1.0f, 1.0f, 0.0f, 0.0f, -1.0f }; Vertices[5] = new Vertex(ptn6);
            float[] ptn7 = { 1.0f, 1.0f, -1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f }; Vertices[6] = new Vertex(ptn7);
            float[] ptn8 = { 1.0f, 1.0f, 1.0f, 1.0f, 0.0f, 1.0f, 0.0f, 0.0f }; Vertices[7] = new Vertex(ptn8);
            float[] ptn9 = { 1.0f, -1.0f, -1.0f, 0.0f, 1.0f, 1.0f, 0.0f, 0.0f }; Vertices[8] = new Vertex(ptn9);
            float[] ptn10 = { 1.0f, -1.0f, -1.0f, 0.0f, 1.0f, 1.0f, 0.0f, 0.0f }; Vertices[9] = new Vertex(ptn10);
            float[] ptn11 = { 1.0f, 1.0f, 1.0f, 1.0f, 0.0f, 1.0f, 0.0f, 0.0f }; Vertices[10] = new Vertex(ptn11);
            float[] ptn12 = { 1.0f, -1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 0.0f, 0.0f }; Vertices[11] = new Vertex(ptn12);
            float[] ptn13 = { 1.0f, 1.0f, 1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f }; Vertices[12] = new Vertex(ptn13);
            float[] ptn14 = { -1.0f, 1.0f, 1.0f, 1.0f, 0.0f, 0.0f, 0.0f, 1.0f }; Vertices[13] = new Vertex(ptn14);
            float[] ptn15 = { 1.0f, -1.0f, 1.0f, 0.0f, 1.0f, 0.0f, 0.0f, 1.0f }; Vertices[14] = new Vertex(ptn15);
            float[] ptn16 = { 1.0f, -1.0f, 1.0f, 0.0f, 1.0f, 0.0f, 0.0f, 1.0f }; Vertices[15] = new Vertex(ptn16);
            float[] ptn17 = { -1.0f, 1.0f, 1.0f, 1.0f, 0.0f, 0.0f, 0.0f, 1.0f }; Vertices[16] = new Vertex(ptn17);
            float[] ptn18 = { -1.0f, -1.0f, 1.0f, 1.0f, 1.0f, 0.0f, 0.0f, 1.0f }; Vertices[17] = new Vertex(ptn18);
            float[] ptn19 = { -1.0f, 1.0f, 1.0f, 0.0f, 0.0f, -1.0f, 0.0f, 0.0f }; Vertices[18] = new Vertex(ptn19);
            float[] ptn20 = { -1.0f, 1.0f, -1.0f, 1.0f, 0.0f, -1.0f, 0.0f, 0.0f }; Vertices[19] = new Vertex(ptn20);
            float[] ptn21 = { -1.0f, -1.0f, 1.0f, 0.0f, 1.0f, -1.0f, 0.0f, 0.0f }; Vertices[20] = new Vertex(ptn21);
            float[] ptn22 = { -1.0f, -1.0f, 1.0f, 0.0f, 1.0f, -1.0f, 0.0f, 0.0f }; Vertices[21] = new Vertex(ptn22);
            float[] ptn23 = { -1.0f, 1.0f, -1.0f, 1.0f, 0.0f, -1.0f, 0.0f, 0.0f }; Vertices[22] = new Vertex(ptn23);
            float[] ptn24 = { -1.0f, -1.0f, -1.0f, 1.0f, 1.0f, -1.0f, 0.0f, 0.0f }; Vertices[23] = new Vertex(ptn24);
            float[] ptn25 = { -1.0f, 1.0f, 1.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f }; Vertices[24] = new Vertex(ptn25);
            float[] ptn26 = { 1.0f, 1.0f, 1.0f, 1.0f, 0.0f, 0.0f, 1.0f, 0.0f }; Vertices[25] = new Vertex(ptn26);
            float[] ptn27 = { -1.0f, 1.0f, -1.0f, 0.0f, 1.0f, 0.0f, 1.0f, 0.0f }; Vertices[26] = new Vertex(ptn27);
            float[] ptn28 = { -1.0f, 1.0f, -1.0f, 0.0f, 1.0f, 0.0f, 1.0f, 0.0f }; Vertices[27] = new Vertex(ptn28);
            float[] ptn29 = { 1.0f, 1.0f, 1.0f, 1.0f, 0.0f, 0.0f, 1.0f, 0.0f }; Vertices[28] = new Vertex(ptn29);
            float[] ptn30 = { 1.0f, 1.0f, -1.0f, 1.0f, 1.0f, 0.0f, 1.0f, 0.0f }; Vertices[29] = new Vertex(ptn30);
            float[] ptn31 = { -1.0f, -1.0f, -1.0f, 0.0f, 0.0f, 0.0f, -1.0f, 0.0f }; Vertices[30] = new Vertex(ptn31);
            float[] ptn32 = { 1.0f, -1.0f, -1.0f, 1.0f, 0.0f, 0.0f, -1.0f, 0.0f }; Vertices[31] = new Vertex(ptn32);
            float[] ptn33 = { -1.0f, -1.0f, 1.0f, 0.0f, 1.0f, 0.0f, -1.0f, 0.0f }; Vertices[32] = new Vertex(ptn33);
            float[] ptn34 = { -1.0f, -1.0f, 1.0f, 0.0f, 1.0f, 0.0f, -1.0f, 0.0f }; Vertices[33] = new Vertex(ptn34);
            float[] ptn35 = { 1.0f, -1.0f, -1.0f, 1.0f, 0.0f, 0.0f, -1.0f, 0.0f }; Vertices[34] = new Vertex(ptn35);
            float[] ptn36 = { 1.0f, -1.0f, 1.0f, 1.0f, 1.0f, 0.0f, -1.0f, 0.0f }; Vertices[35] = new Vertex(ptn36);

            foreach (Vertex v in Vertices)
            {
                v.binormal = new Vector3(0.0f, 0.0f, 0.0f);
                v.tangent = new Vector3(0.0f, 0.0f, 0.0f);
                v.normal = new Vector3(0.0f, 0.0f, 0.0f);
            }

            WorldPositionFace = new Face[0];
            WorldVertics = new Vertex[0];
            CalculateNormalTangentBinormal();


            int TotalVertices = 14 * vertexCount;
            vertices = new float[TotalVertices];
            //{  
            //        -1.0f,  1.0f, -0.2f, 0.0f, 0.0f , 0.0f , 0.0f,-1.0f,1.0f,0.0f,0.0f,0.0f,-1.0f,0.0f,
            //         1.0f, 1.0f ,-0.2f,  1.0f, 0.0f , 0.0f  ,0.0f,-1.0f,1.0f,0.0f,0.0f,0.0f,-1.0f,0.0f,
            //        -1.0f, -1.0f ,-0.2f ,0.0f ,1.0f , 0.0f , 0.0f, -1.0f,1.0f,0.0f,0.0f,0.0f,-1.0f,0.0f,
            //        -1.0f ,-1.0f ,-0.2f ,0.0f, 1.0f , 0.0f , 0.0f ,-1.0f,1.0f,0.0f,0.0f,0.0f,-1.0f,0.0f,
            //         1.0f , 1.0f, -0.2f ,1.0f ,0.0f , 0.0f , 0.0f ,-1.0f,1.0f,0.0f,0.0f,0.0f,-1.0f,0.0f,
            //         1.0f,  -1.0f, -0.2f ,1.0f, 1.0f , 0.0f  ,0.0f ,-1.0f,1.0f,0.0f,0.0f,0.0f,-1.0f,0.0f

            //};
            int vindex = 0;
            for (int i = 0; i < vertexCount; i++)
            {
                vertices[vindex] = Vertices[i].pos.X;
                vindex++;
                vertices[vindex] = Vertices[i].pos.Y;
                vindex++;
                vertices[vindex] = Vertices[i].pos.Z;
                vindex++;
                vertices[vindex] = Vertices[i].tex.X;
                vindex++;
                vertices[vindex] = Vertices[i].tex.Y;
                vindex++;
                vertices[vindex] = Vertices[i].normal.X;
                vindex++;
                vertices[vindex] = Vertices[i].normal.Y;
                vindex++;
                vertices[vindex] = Vertices[i].normal.Z;
                vindex++;
                vertices[vindex] = Vertices[i].tangent.X;
                vindex++;
                vertices[vindex] = Vertices[i].tangent.Y;
                vindex++;
                vertices[vindex] = Vertices[i].tangent.Z;
                vindex++;
                vertices[vindex] = Vertices[i].binormal.X;
                vindex++;
                vertices[vindex] = Vertices[i].binormal.Y;
                vindex++;
                vertices[vindex] = Vertices[i].binormal.Z;
                vindex++;
            }
            //verticesPosition = new Vector3[]
            //    { 
            //        new Vector3(-1.0f,  1.0f, -0.2f),
            //        new Vector3(1.0f, 1.0f ,-0.2f),
            //        new Vector3(-1.0f, -1.0f ,-0.2f),
            //        new Vector3(-1.0f ,-1.0f ,-0.2f),
            //        new Vector3(1.0f , 1.0f, -0.2f),
            //        new Vector3(1.0f,  -1.0f, -0.2f)
            //    };
            verticesPosition = new Vector3[vertexCount];
            for (int i = 0; i < vertexCount; i++)
            {
                verticesPosition[i] = Vertices[i].pos;
            }

            //short[] faces = new[]
            //                    {
            //                            (short)0, (short)1, (short)2,
            //                            (short)3, (short)4, (short)5
            //                    };

            int IndexCount = vertexCount;
            short[] faces = new short[IndexCount];
            for (int i = 0; i < IndexCount; i++)
            {
                faces[i] = (short)i;
            }

            // Creating vertex buffer
            vertexSize = sizeof(float) * 14;
            var stream = new DataStream(vertexCount * vertexSize, true, true);
            stream.WriteRange(vertices);
            stream.Position = 0;

            vertexBuffer = new SlimDX.Direct3D11.Buffer(device, stream, new BufferDescription
            {
                BindFlags = BindFlags.VertexBuffer,
                CpuAccessFlags = CpuAccessFlags.None,
                OptionFlags = ResourceOptionFlags.None,
                SizeInBytes = (int)stream.Length,
                Usage = ResourceUsage.Default
            });
            stream.Dispose();

            // Index buffer
            stream = new DataStream(IndexCount * sizeof(short), true, true);
            stream.WriteRange(faces);
            stream.Position = 0;

            var indices = new SlimDX.Direct3D11.Buffer(device, stream, new BufferDescription
            {
                BindFlags = BindFlags.IndexBuffer,
                CpuAccessFlags = CpuAccessFlags.None,
                OptionFlags = ResourceOptionFlags.None,
                SizeInBytes = (int)stream.Length,
                Usage = ResourceUsage.Default
            });
            stream.Dispose();
            Faces = faces;
            // Uploading to the device
            device.ImmediateContext.InputAssembler.PrimitiveTopology = PrimitiveTopology.TriangleList;
            device.ImmediateContext.InputAssembler.SetVertexBuffers(0, new VertexBufferBinding(vertexBuffer, vertexSize, 0));
            device.ImmediateContext.InputAssembler.SetIndexBuffer(indices, SlimDX.DXGI.Format.R16_UInt, 0);
        }
        public void UpdateGeometry()
        {
            for (int i = 0; i < PurePosition.Length; i++)
            {
                vertices[i * 14]=PurePosition[i].X;
                vertices[i * 14 + 1]=PurePosition[i].Y;
                vertices[i * 14 + 2]=PurePosition[i].Z;
            }

            if (!IsRigidBody)
            {
                for (int i = 0; i < PurePositionWorldPosition.Length; i++)
                {

                    PurePositionWorldPosition[i] = Multiply(PurePosition[i], worldMatrix);

                }
            }

        }
        public void SetGeometry(Vertex[] V, short[] faces,Device device,Face[] positionFace)
        {
            


            Faces = faces;
            float Max=0;
            for(int i=0;i<V.Length;i++)
            {
                 float r=V[i].pos.X*Scale.X*V[i].pos.X*Scale.X+V[i].pos.Y*Scale.Y*V[i].pos.Y*Scale.Y+V[i].pos.Z*Scale.Z*V[i].pos.Z*Scale.Z;
                 if(Max<r)
                 {
                     Max=r;
                 }
                
            }
            Radius = Max;
            Radius = (float)Math.Sqrt(Radius);

            Vertices = V;
            WorldVertics = new Vertex[V.Length];

            for (int i = 0; i < WorldVertics.Length; i++)
            {
                WorldVertics[i] = new Vertex();
            }
            

            PositionFace = new Face[positionFace.Length];
            PositionFace = positionFace;


            WorldPositionFace = new Face[positionFace.Length];
            for (int i = 0; i < WorldPositionFace.Length;i++ )
            {
                WorldPositionFace[i] = new Face();
            }
           
            UpdateFaces();


            vertexCount = faces.Length;
            int TotalVertices = 14 * V.Length;
            vertices = new float[TotalVertices];// Here we 
            int vindex = 0;
            for (int i = 0; i < V.Length; i++)
            {
                vertices[vindex] = V[i].pos.X;
                vindex++;
                vertices[vindex] = V[i].pos.Y;
                vindex++;
                vertices[vindex] = V[i].pos.Z;
                vindex++;
                vertices[vindex] = V[i].tex.X;
                vindex++;
                vertices[vindex] = V[i].tex.Y;
                vindex++;
                vertices[vindex] = V[i].normal.X;
                vindex++;
                vertices[vindex] = V[i].normal.Y;
                vindex++;
                vertices[vindex] = V[i].normal.Z;
                vindex++;
                vertices[vindex] = V[i].tangent.X;
                vindex++;
                vertices[vindex] = V[i].tangent.Y;
                vindex++;
                vertices[vindex] = V[i].tangent.Z;
                vindex++;
                vertices[vindex] = V[i].binormal.X;
                vindex++;
                vertices[vindex] = V[i].binormal.Y;
                vindex++;
                vertices[vindex] = V[i].binormal.Z;
                vindex++;
            }

            vertexSize = sizeof(float) * 14;
            var stream = new DataStream(vertices.Length * vertexSize, true, true);
            stream.WriteRange(vertices);
            stream.Position = 0;

            vertexBuffer = new SlimDX.Direct3D11.Buffer(device, stream, new BufferDescription
            {
                BindFlags = BindFlags.VertexBuffer,
                CpuAccessFlags = CpuAccessFlags.None,
                OptionFlags = ResourceOptionFlags.None,
                SizeInBytes = (int)stream.Length,
                Usage = ResourceUsage.Default
            });
            stream.Dispose();

            // Index buffer
            stream = new DataStream(faces.Length * sizeof(short), true, true);
            stream.WriteRange(faces);
            stream.Position = 0;

            var indices = new SlimDX.Direct3D11.Buffer(device, stream, new BufferDescription
            {
                BindFlags = BindFlags.IndexBuffer,
                CpuAccessFlags = CpuAccessFlags.None,
                OptionFlags = ResourceOptionFlags.None,
                SizeInBytes = (int)stream.Length,
                Usage = ResourceUsage.Default
            });
            stream.Dispose();
            PurePosition = new Vector3[vertices.Length/14];
            PurePositionWorldPosition = new Vector3[vertices.Length / 14];

            for (int i = 0; i < PurePosition.Length; i++)
            {
                  PurePosition[i].X=vertices[i * 14];
                  PurePosition[i].Y=vertices[i * 14 + 1];
                  PurePosition[i].Z=vertices[i * 14 + 2];
            }
            // Uploading to the device
            device.ImmediateContext.InputAssembler.PrimitiveTopology = PrimitiveTopology.TriangleList;
            device.ImmediateContext.InputAssembler.SetVertexBuffers(0, new VertexBufferBinding(vertexBuffer, vertexSize, 0));
            device.ImmediateContext.InputAssembler.SetIndexBuffer(indices, SlimDX.DXGI.Format.R16_UInt, 0);
        }
        public void UpdateVertices()
        {
            
            for (int i = 0; i < WorldVertics.Length; i++)
            {
                WorldVertics[i].pos = Multiply(Vertices[i].pos, worldMatrix);
                WorldVertics[i].normal = Multiply(Vertices[i].normal, Matrix.RotationQuaternion(quaternion));
            }
        }
        public void UpdateFaces()
        {
           
            for (int i = 0; i < WorldPositionFace.Length; i++)
            {
                WorldPositionFace[i].v1 = Multiply(PositionFace[i].v1, worldMatrix);
                WorldPositionFace[i].v2 = Multiply(PositionFace[i].v2, worldMatrix);
                WorldPositionFace[i].v3 = Multiply(PositionFace[i].v3, worldMatrix);
                WorldPositionFace[i].center = (WorldPositionFace[i].v1 + WorldPositionFace[i].v2 + WorldPositionFace[i].v3) / 3;
                WorldPositionFace[i].position = Multiply(PositionFace[i].position, worldMatrix);
                WorldPositionFace[i].normal = Multiply(PositionFace[i].normal, worldMatrix);
            }
          
        }

        public void UpdateVertices(Device device)
        {

            var stream = new DataStream(vertexCount * vertexSize, true, true);
            stream.WriteRange(vertices);
            stream.Position = 0;
            vertexBuffer = new SlimDX.Direct3D11.Buffer(device, stream, new BufferDescription
            {
                BindFlags = BindFlags.VertexBuffer,
                CpuAccessFlags = CpuAccessFlags.None,
                OptionFlags = ResourceOptionFlags.None,
                SizeInBytes = (int)stream.Length,
                Usage = ResourceUsage.Default
            });
            stream.Dispose();

            // Index buffer
            stream = new DataStream(Faces.Length * sizeof(short), true, true);
            stream.WriteRange(Faces);
            stream.Position = 0;

            var indices = new SlimDX.Direct3D11.Buffer(device, stream, new BufferDescription
            {
                BindFlags = BindFlags.IndexBuffer,
                CpuAccessFlags = CpuAccessFlags.None,
                OptionFlags = ResourceOptionFlags.None,
                SizeInBytes = (int)stream.Length,
                Usage = ResourceUsage.Default
            });
            stream.Dispose();
          
            device.ImmediateContext.InputAssembler.PrimitiveTopology = PrimitiveTopology.TriangleList;
            device.ImmediateContext.InputAssembler.SetVertexBuffers(0, new VertexBufferBinding(vertexBuffer, vertexSize, 0));
            
            device.ImmediateContext.InputAssembler.SetIndexBuffer(indices, SlimDX.DXGI.Format.R16_UInt, 0);

            vertexBuffer.Dispose();
            indices.Dispose();
        }
        public void LoadModel(Device device)
        {
            

            //short[] faces = new[]
            //                    {
            //                            (short)0, (short)1, (short)2,
            //                            (short)3, (short)4, (short)5
            //                    };
            vertexCount = 36;
            int IndexCount = vertexCount;
            short[] faces = new short[IndexCount];
            for (int i = 0; i < IndexCount; i++)
            {
                faces[i] = (short)i;
            }

            // Creating vertex buffer
            const int vertexSize = sizeof(float) * 14;
            var stream = new DataStream(vertexCount * vertexSize, true, true);
            stream.WriteRange(vertices);
            stream.Position = 0;

            var vertexBuffer = new SlimDX.Direct3D11.Buffer(device, stream, new BufferDescription
            {
                BindFlags = BindFlags.VertexBuffer,
                CpuAccessFlags = CpuAccessFlags.None,
                OptionFlags = ResourceOptionFlags.None,
                SizeInBytes = (int)stream.Length,
                Usage = ResourceUsage.Default
            });
            stream.Dispose();

            // Index buffer
            stream = new DataStream(IndexCount * sizeof(short), true, true);
            stream.WriteRange(faces);
            stream.Position = 0;

            var indices = new SlimDX.Direct3D11.Buffer(device, stream, new BufferDescription
            {
                BindFlags = BindFlags.IndexBuffer,
                CpuAccessFlags = CpuAccessFlags.None,
                OptionFlags = ResourceOptionFlags.None,
                SizeInBytes = (int)stream.Length,
                Usage = ResourceUsage.Default
            });
            stream.Dispose();

            // Uploading to the device
            device.ImmediateContext.InputAssembler.PrimitiveTopology = PrimitiveTopology.TriangleList;
            device.ImmediateContext.InputAssembler.SetVertexBuffers(0, new VertexBufferBinding(vertexBuffer, vertexSize, 0));
            device.ImmediateContext.InputAssembler.SetIndexBuffer(indices, SlimDX.DXGI.Format.R16_UInt, 0);
            
        }
        public void CalculateNormalTangentBinormal()
        {
            int faceCount, i, index;
            TempVertexType vertex1, vertex2, vertex3;
            Vector3 tangent=new Vector3(0.0f,0.0f,0.0f);
            Vector3 binormal=new Vector3(0.0f,0.0f,0.0f);
            Vector3 normal=new Vector3(0.0f,0.0f,0.0f);
             


            // Calculate the number of faces in the model.
            faceCount = vertexCount / 3;

            // Initialize the index to the model data.
            index = 0;

            // Go through all the faces and calculate the the tangent, binormal, and normal vectors.
            for (i = 0; i < faceCount; i++)
            {
                // Get the three vertices for this face from the model.
                vertex1.x = Vertices[index].pos.X;
                vertex1.y = Vertices[index].pos.Y;
                vertex1.z = Vertices[index].pos.Z;
                vertex1.tu = Vertices[index].tex.X;
                vertex1.tv = Vertices[index].tex.Y;
                vertex1.nx = Vertices[index].normal.X;
                vertex1.ny = Vertices[index].normal.Y;
                vertex1.nz = Vertices[index].normal.Z;
                index++;

                vertex2.x = Vertices[index].pos.X;
                vertex2.y = Vertices[index].pos.Y;
                vertex2.z = Vertices[index].pos.Z;
                vertex2.tu = Vertices[index].tex.X;
                vertex2.tv = Vertices[index].tex.Y;
                vertex2.nx = Vertices[index].normal.X;
                vertex2.ny = Vertices[index].normal.Y;
                vertex2.nz = Vertices[index].normal.Z;
                index++;

                vertex3.x = Vertices[index].pos.X;
                vertex3.y = Vertices[index].pos.Y;
                vertex3.z = Vertices[index].pos.Z;
                vertex3.tu = Vertices[index].tex.X;
                vertex3.tv = Vertices[index].tex.Y;
                vertex3.nx = Vertices[index].normal.X;
                vertex3.ny = Vertices[index].normal.Y;
                vertex3.nz = Vertices[index].normal.Z;
                index++;

                // Calculate the tangent and binormal of that face.
                CalculateTangentBinormal(vertex1, vertex2, vertex3, ref tangent,ref  binormal);

                // Calculate the new normal using the tangent and binormal.
                CalculateNormal(tangent, binormal, ref normal);

                // Store the normal, tangent, and binormal for this face back in the model structure.
                Vertices[index - 1].normal.X = normal.X;
                Vertices[index - 1].normal.Y = normal.Y;
                Vertices[index - 1].normal.Z = normal.Z;
                Vertices[index - 1].tangent.X = tangent.X;
                Vertices[index - 1].tangent.Y = tangent.Y;
                Vertices[index - 1].tangent.Z = tangent.Z;
                Vertices[index - 1].binormal.X = binormal.X;
                Vertices[index - 1].binormal.Y = binormal.Y;
                Vertices[index - 1].binormal.Z = binormal.Z;

                Vertices[index - 2].normal.X = normal.X;
                Vertices[index - 2].normal.Y = normal.Y;
                Vertices[index - 2].normal.Z = normal.Z;
                Vertices[index - 2].tangent.X = tangent.X;
                Vertices[index - 2].tangent.Y = tangent.Y;
                Vertices[index - 2].tangent.Z = tangent.Z;
                Vertices[index - 2].binormal.X = binormal.X;
                Vertices[index - 2].binormal.Y = binormal.Y;
                Vertices[index - 2].binormal.Z = binormal.Z;

                Vertices[index - 3].normal.X = normal.X;
                Vertices[index - 3].normal.Y = normal.Y;
                Vertices[index - 3].normal.Z = normal.Z;
                Vertices[index - 3].tangent.X = tangent.X;
                Vertices[index - 3].tangent.Y = tangent.Y;
                Vertices[index - 3].tangent.Z = tangent.Z;
                Vertices[index - 3].binormal.X = binormal.X;
                Vertices[index - 3].binormal.Y = binormal.Y;
                Vertices[index - 3].binormal.Z = binormal.Z;

            }
            
        }
        
        
        public void CalculateTangentBinormal(TempVertexType vertex1, TempVertexType vertex2, TempVertexType vertex3, ref Vector3 tangent, ref Vector3 binormal)
        {
            float []vector1=new float[3];
            float []vector2=new float[3];
            float []tuVector = new float[2];
            float []tvVector=new float[2];
	        float den;
            float length;


	        // Calculate the two vectors for this face.
            vector1[0] = vertex2.x - vertex1.x;
            vector1[1] = vertex2.y - vertex1.y;
            vector1[2] = vertex2.z - vertex1.z;

            vector2[0] = vertex3.x - vertex1.x;
            vector2[1] = vertex3.y - vertex1.y;
            vector2[2] = vertex3.z - vertex1.z;
	        // Calculate the tu and tv texture space vectors.
            tuVector[0] = vertex2.tu - vertex1.tu;
            tvVector[0] = vertex2.tv - vertex1.tv;

            tuVector[1] = vertex3.tu - vertex1.tu;
            tvVector[1] = vertex3.tv - vertex1.tv;
         


	        // Calculate the denominator of the tangent/binormal equation.
	        den = 1.0f / (tuVector[0] * tvVector[1] - tuVector[1] * tvVector[0]);

	        // Calculate the cross products and multiply by the coefficient to get the tangent and binormal.
	        tangent.X = (tvVector[1] * vector1[0] - tvVector[0] * vector2[0]) * den;
	        tangent.Y = (tvVector[1] * vector1[1] - tvVector[0] * vector2[1]) * den;
	        tangent.Z = (tvVector[1] * vector1[2] - tvVector[0] * vector2[2]) * den;

	        binormal.X = (tuVector[0] * vector2[0] - tuVector[1] * vector1[0]) * den;
	        binormal.Y = (tuVector[0] * vector2[1] - tuVector[1] * vector1[1]) * den;
	        binormal.Z = (tuVector[0] * vector2[2] - tuVector[1] * vector1[2]) * den;

            //// Calculate the length of this normal.
            //length = tangent.Length();
			
            //// Normalize the normal and then store it
            //tangent.X = (float)(tangent.X / length);
            //tangent.Y = (float)(tangent.Y / length);
            //tangent.Z = (float)(tangent.Z / length);
            tangent.Normalize();

	        // Calculate the length of this normal.
            //length = (float)Math.Sqrt((binormal.X * binormal.X) + (binormal.Y * binormal.Y) + (binormal.Z * binormal.Z));
			
            //// Normalize the normal and then store it
            //binormal.X = (float)(binormal.X / length);
            //binormal.Y = (float)(binormal.Y / length);
            //binormal.Z = (float)(binormal.Z / length);

            binormal.Normalize();
        }
        public void CalculateNormal(Vector3 tangent, Vector3 binormal, ref Vector3 normal)
        {
	        float length;

	        // Calculate the cross product of the tangent and binormal which will give the normal vector.
	        normal.X = (tangent.Y * binormal.Z) - (tangent.Z * binormal.Y);
	        normal.Y = (tangent.Z * binormal.X) - (tangent.X * binormal.Z);
	        normal.Z = (tangent.X * binormal.Y) - (tangent.Y * binormal.X);

	        // Calculate the length of the normal.
	        length = (float)Math.Sqrt((normal.X * normal.X) + (normal.Y * normal.Y) + (normal.Z * normal.Z));
            
	        // Normalize the normal.
	        normal.X = normal.X / length;
	        normal.Y = normal.Y / length;
	        normal.Z = normal.Z / length;

	        return;
        }
        
        public void createSphere()
        {

        }

    }
}
