﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SlimDX;
using SlimDX.D3DCompiler;
using SlimDX.Direct3D11;
using SlimDX.DXGI;
using Device = SlimDX.Direct3D11.Device;
using Resource = SlimDX.Direct3D11.Resource;
namespace LittleBigTable
{
    //We use Camera to generate ViewMatrix and ProjectionMatrix;
    public class Camera
    {
        public Vector3 Position;

        public Vector3 LookAt;
        Vector3 Up;
        public Matrix ViewMatrix;
        public Matrix ProjectionMatrix;
        float Fov;
        int Width;
        int Height;
        float Near;
        float Far;
        Matrix rotation;
        Matrix translation;
        public Camera()
        {
        }
        public Camera(Camera camera)
        {
            Position=camera.Position;
            LookAt = camera.LookAt;
            Up = camera.Up;
            ViewMatrix = camera.ViewMatrix;
            ProjectionMatrix = camera.ProjectionMatrix;
            Fov = camera.Fov;
            Width = camera.Width;
            Height = camera.Height;
            Near = camera.Near;
            Far = camera.Far;
            
        }
        public int GetWidth()
        {
            return Width;
        }
        public int GetHeight()
        {
            return Height;
        }
        
        public void InitializeCamera(Vector3 posValue, Vector3 targetValue, Vector3 upValue, float FovValue,
            int WidthValue ,int HeightValue,float NearValue,float FarValue)
        {
            Position = posValue;
            LookAt = targetValue;
            Up = upValue;
            Fov = FovValue;
            Width = WidthValue;
            Height = HeightValue;
            Near = NearValue;
            Far = FarValue;
            Matrix.LookAtLH(ref Position, ref LookAt, ref Up, out ViewMatrix);
            rotation = Matrix.Identity;
            UpdateCamera();
            
        }
        public void SetProjectionValues(float FovValue,int WidthValue
            ,int HeightValue,float NearValue,float FarValue)
        {
             Fov = FovValue;
            Width = WidthValue;
            Height = HeightValue;
            Near = NearValue;
            Far = FarValue;
            UpdateCamera();
        }
        public void SetPosition(Vector3 posValue)
        {
            Position = posValue;
        }
        public void SetLookAt(Vector3 targetValue)
        {
            LookAt = targetValue;
        }
        public void SetUp(Vector3 upValue)
        {
            Up = upValue;
        }
        public Vector3 GetPosition()
        {
            
            return Position;
        }
        public Vector3 GetLookAt()
        {
            return LookAt;
        }
        public Vector3 GetUpAxis()
        {
            return Up;
        }
        public Matrix GetViewMatrix()
        {
            return ViewMatrix;
        }
        public Matrix GetProjectionMatrix()
        {
            return ProjectionMatrix;
        }
        public void UpdateCamera()
        {

           // Matrix.LookAtLH(ref Position, ref LookAt, ref Up, out ViewMatrix);
         
            ProjectionMatrix=Matrix.PerspectiveFovLH(Fov, (float)(Width)/(float)(Height), Near, Far);
            
      
        }
        public void MoveForward(float scale)
        {
            LookAt = Vector3.TransformCoordinate(Vector3.UnitZ, rotation);


            ViewMatrix = ViewMatrix * Matrix.Translation(scale*LookAt);
        }
        public void Rotation(float y,float p,float r)
        {
            rotation = Matrix.RotationYawPitchRoll(y, p, r);
            ViewMatrix = ViewMatrix * rotation;


           // right = Vector3.TransformCoordinate(Vector3.UnitX, rotation);
            //up = Vector3.TransformCoordinate(Vector3.UnitY, rotation);
           
            
           
            //Up = Multiply(Up, Matrix.RotationYawPitchRoll(y, p, r));
            //Up.Normalize();
            //Matrix.LookAtLH(ref Position, ref LookAt, ref Up, out ViewMatrix);

        }
        public void Translation(float x, float y, float z)
        {
            ViewMatrix = Matrix.Multiply(ViewMatrix, Matrix.Translation(x, y, z));
        }
        public Vector3 Multiply(Vector3 v, Matrix rotation)
        {
            float x = v.X * rotation.get_Columns(0).X + v.Y * rotation.get_Columns(0).Y + v.Z * rotation.get_Columns(0).Z;
            float y = v.X * rotation.get_Columns(1).X + v.Y * rotation.get_Columns(1).Y + v.Z * rotation.get_Columns(1).Z ;
            float z = v.X * rotation.get_Columns(2).X + v.Y * rotation.get_Columns(2).Y + v.Z * rotation.get_Columns(2).Z ;

            return new Vector3(x,y,z);
        }
        public void SetWidthHeight(int width, int height)
        {
            Width = width;
            Height = height;
        }
    }
}
