﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SlimDX;
using SlimDX.D3DCompiler;
using SlimDX.Direct3D11;
using SlimDX.DXGI;
using Device = SlimDX.Direct3D11.Device;
using Resource = SlimDX.Direct3D11.Resource;

namespace LittleBigTable
{

    public class D3DDevice
    {
        Device device11;
        float Width;
        float Height;
        Factory1 factory1;
        Adapter1 adapter1;
        public SlimDX.DirectInput.Keyboard keyboard;
        public SlimDX.DirectInput.KeyboardState keyboradState;

        SwapChain swapChain;
        SlimDX.DirectInput.DirectInput input;
        SlimDX.DirectInput.Mouse mouse;
        SlimDX.DirectInput.MouseState mouseState;
        Texture2D backBuffer;
        RenderTargetView renderTargetView;
        DepthStencilView depthStencilView;
        DepthStencilState dssState;
        DepthStencilView ShadowDepthStencilView;
        ShaderResourceView ShadowShaderResourceView;
        Texture2D ShadowTexture;
        public bool CreateDevice(System.IntPtr Handle, int width, int height)
        {

            // Creating device (we accept dx10 cards or greater)
            FeatureLevel[] levels = {
                                        FeatureLevel.Level_11_0,
                                        FeatureLevel.Level_10_1,
                                        FeatureLevel.Level_10_0
                                    };
            Width = width;
            Height = height;
            // Defining our swap chain
            SwapChainDescription desc = new SwapChainDescription();
            desc.BufferCount = 2;
            desc.Usage = Usage.BackBuffer | Usage.RenderTargetOutput;
            desc.ModeDescription = new ModeDescription(0, 0, new Rational(0, 0), Format.R8G8B8A8_UNorm);
            desc.SampleDescription = new SampleDescription(4, 0);
            desc.OutputHandle = Handle;
            desc.IsWindowed = true;
            desc.SwapEffect = SwapEffect.Discard;
            Device.CreateWithSwapChain(DriverType.Hardware, DeviceCreationFlags.None, levels, desc, out device11, out swapChain);
            // Getting back buffer
            backBuffer = Resource.FromSwapChain<Texture2D>(swapChain, 0);
            // Defining render view
            renderTargetView = new RenderTargetView(device11, backBuffer);
            // device11.ImmediateContext.OutputMerger.SetTargets(renderTargetView);
            //Defining Depth Buffer
            backBuffer.Dispose();
            SampleDescription sampleDesc = new SlimDX.DXGI.SampleDescription();
            sampleDesc.Count = 4;
            sampleDesc.Quality = 0;

            Texture2DDescription texDesc = new Texture2DDescription();
            texDesc.Width = width;
            texDesc.Height = height;
            texDesc.MipLevels = 1;
            texDesc.ArraySize = 1;
            texDesc.BindFlags = BindFlags.DepthStencil;
            texDesc.CpuAccessFlags = CpuAccessFlags.None;
            texDesc.Format = Format.D24_UNorm_S8_UInt;
            texDesc.SampleDescription = sampleDesc;

            Texture2DDescription depthdesc = new Texture2DDescription();
            depthdesc.BindFlags = BindFlags.DepthStencil;
            depthdesc.Format = Format.D32_Float_S8X24_UInt;
            depthdesc.Width = width;
            depthdesc.Height = height;
            depthdesc.MipLevels = 1;
            depthdesc.SampleDescription = sampleDesc;
            depthdesc.Usage = ResourceUsage.Default;
            depthdesc.OptionFlags = ResourceOptionFlags.None;
            depthdesc.CpuAccessFlags = CpuAccessFlags.None;
            depthdesc.ArraySize = 1;




            DepthStencilViewDescription stencilViewDesc = new DepthStencilViewDescription();
            stencilViewDesc.Dimension = DepthStencilViewDimension.Texture2DMultisampled;
            stencilViewDesc.Format = texDesc.Format;
            stencilViewDesc.MipSlice = 0;
            //  stencilViewDesc.FirstArraySlice = 0;        
            // stencilViewDesc.ArraySize = 1;

            using (Texture2D stencilTex = new Texture2D(device11, texDesc))
            {
                depthStencilView = new DepthStencilView(device11, stencilTex, stencilViewDesc);
            }
            DepthStencilStateDescription stencilStateDesc = new DepthStencilStateDescription();
            stencilStateDesc.IsDepthEnabled = true;
            stencilStateDesc.IsStencilEnabled = false;
            stencilStateDesc.DepthWriteMask = DepthWriteMask.All;
            stencilStateDesc.DepthComparison = Comparison.Less;
            stencilStateDesc.StencilReadMask = 0xFF;
            stencilStateDesc.StencilWriteMask = 0xFF;

            DepthStencilOperationDescription dsod = new DepthStencilOperationDescription();
            dsod.FailOperation = StencilOperation.Keep;
            dsod.DepthFailOperation = StencilOperation.Increment;
            dsod.PassOperation = StencilOperation.Keep;
            dsod.Comparison = Comparison.Always;
            stencilStateDesc.FrontFace = dsod;
            dsod.DepthFailOperation = StencilOperation.Decrement;
            stencilStateDesc.BackFace = dsod;
            dssState = DepthStencilState.FromDescription(device11, stencilStateDesc);
            device11.ImmediateContext.OutputMerger.DepthStencilState = dssState;

            // device11.ImmediateContext.OutputMerger.
            //device11.ImmediateContext.OutputMerger.SetTargets
            device11.ImmediateContext.OutputMerger.SetTargets(depthStencilView, renderTargetView);
            device11.ImmediateContext.Rasterizer.SetViewports(new Viewport(0, 0, width, height, 0.0f, 1.0f));
            //Create Mouse and MouseState
            input = new SlimDX.DirectInput.DirectInput();
            mouse = new SlimDX.DirectInput.Mouse(input);
            mouse.Acquire();
            mouseState = new SlimDX.DirectInput.MouseState();
            mouse.GetCurrentState(ref mouseState);
            //Create Adapter
            factory1 = new Factory1();
            adapter1 = factory1.GetAdapter1(0);


            keyboard = new SlimDX.DirectInput.Keyboard(input);
            keyboard.Acquire();
            keyboradState = new SlimDX.DirectInput.KeyboardState();
            keyboard.GetCurrentState(ref keyboradState);


            RasterizerStateDescription rasterizerStateDescription = new RasterizerStateDescription
            {
                CullMode = CullMode.None,
                FillMode = FillMode.Solid,
                IsAntialiasedLineEnabled = true,
                // IsScissorEnabled=true
            };
            device11.ImmediateContext.Rasterizer.State = RasterizerState.FromDescription(device11, rasterizerStateDescription);
            //Create Shadow Map Texture
            Texture2DDescription shadowTextureDsc = new Texture2DDescription();
            shadowTextureDsc.Width = width;
            shadowTextureDsc.Height = height;
            shadowTextureDsc.MipLevels = 1;
            shadowTextureDsc.ArraySize = 1;
            shadowTextureDsc.Format = Format.R32_Typeless;
            shadowTextureDsc.SampleDescription = sampleDesc;
            shadowTextureDsc.Usage = ResourceUsage.Default;
            shadowTextureDsc.BindFlags = BindFlags.DepthStencil | BindFlags.ShaderResource;
            shadowTextureDsc.CpuAccessFlags = CpuAccessFlags.None;

            DepthStencilViewDescription dsvDsc = new DepthStencilViewDescription();
            dsvDsc.Format = Format.D32_Float;
            dsvDsc.Dimension = DepthStencilViewDimension.Texture2DMultisampled;
            dsvDsc.MipSlice = 0;

            ShaderResourceViewDescription srvDsc = new ShaderResourceViewDescription();
            srvDsc.Format = Format.R32_Float;
            srvDsc.Dimension = ShaderResourceViewDimension.Texture2DMultisampled;
            srvDsc.MipLevels = shadowTextureDsc.MipLevels;
            srvDsc.MostDetailedMip = 0;

            ShadowTexture = new Texture2D(device11, shadowTextureDsc);
            ShadowDepthStencilView = new DepthStencilView(device11, ShadowTexture, dsvDsc);
            ShadowShaderResourceView = new ShaderResourceView(device11, ShadowTexture, srvDsc);

            return true;
        }
        public Device GetDevice()
        {
            return device11;
        }
        public void EndScene()
        {
            //Get the state of mouse
            mouse.GetCurrentState(ref mouseState);
            keyboard.GetCurrentState(ref keyboradState);
            //Present and Clear 
            swapChain.Present(0, PresentFlags.None);

        }
        public void UnBindMap(Effect effect)
        {
            effect.GetVariableByName("shadowMap").AsResource().SetResource(null);
            effect.GetTechniqueByName("ShadowMapRenderPassFrontFaces").GetPassByName("P0").Apply(device11.ImmediateContext);
        }
        public void BeginScene(Effect effect)
        {
            //Render Final Scene
            device11.ImmediateContext.OutputMerger.DepthStencilState = dssState;
            device11.ImmediateContext.OutputMerger.SetTargets(depthStencilView,renderTargetView); 
            device11.ImmediateContext.Rasterizer.SetViewports(new Viewport(0, 0, Width, Height, 0.0f, 1.0f));
            device11.ImmediateContext.ClearDepthStencilView(depthStencilView, DepthStencilClearFlags.Depth, 1.0f, 0);
            device11.ImmediateContext.ClearRenderTargetView(renderTargetView, new Color4(0.0f, 0.0f, 0.0f));
            //Bind shadow map texture
            //effect.GetVariableByName("shadowMap").AsResource().SetResource(ShadowShaderResourceView);




        }
        public void prevScene()
        {
            //Set Render Target
            //device11.ImmediateContext.OutputMerger.SetTargets(ShadowDepthStencilView);
            //device11.ImmediateContext.ClearDepthStencilView(ShadowDepthStencilView, DepthStencilClearFlags.Depth, 1.0f, 0);
            //Next Render the scene
        }
        public Adapter1 GetAdapter()
        {
            return adapter1;
        }
        public void Dispose()
        {
            input.Dispose();
            mouse.Dispose();
            renderTargetView.Dispose();
            backBuffer.Dispose();
            swapChain.Dispose();
            device11.Dispose();
            device11 = null;
        }
        public SlimDX.DirectInput.MouseState GetMouseState()
        {
            return mouseState;
        }
        public void Resize(int width, int height)
        {
            //Resize the back buffer
            // backBuffer.Dispose();         
            //Resize the Camera
            // renderTargetView.Dispose();
            renderTargetView.Dispose();
            swapChain.ResizeBuffers(1, width, height, Format.R8G8B8A8_UNorm, SwapChainFlags.None);
            using (var bb = Resource.FromSwapChain<Texture2D>(swapChain, 0))
                renderTargetView = new RenderTargetView(device11, bb);

            device11.ImmediateContext.OutputMerger.SetTargets(depthStencilView, renderTargetView);
            device11.ImmediateContext.Rasterizer.SetViewports(new Viewport(0, 0, width, height, 0.0f, 1.0f));


        }



    }
}