﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SlimDX;
using SlimDX.D3DCompiler;
using SlimDX.Direct3D11;
using SlimDX.DXGI;
using FarseerPhysics.Collision;
using FarseerPhysics.Common;
using FarseerPhysics.Controllers;
using FarseerPhysics.Dynamics;
using FarseerPhysics.Factories;

namespace LittleBigTable
{
    
    using Vec2 = Microsoft.Xna.Framework.Vector2;
    using Device = SlimDX.Direct3D11.Device;
    using Resource = SlimDX.Direct3D11.Resource;
    public enum ObjectUIStatus {Pressed,Released,Touched,Moved}
   
    public class InitializeInf
    {
        public Vector2 Scale;
        public Vector2 Position;
        public float Angle;
        public Vector3 ShadowPosition;
        public float ObjectDepth;
        public float ShadowDepth;
        public float Alpha;
        public float BlurExtended;
        public int Index;
        public bool Pickable;
        public float BumpDensity;
        public float Density;
        public float ShadowDarkness;
        public Vector2 TextLocalPosition;
        public string  colorTextureFileName;
        public  string normalTextureFileName;
        
      
        public void CreateObjectInf(Vector2 position, Vector3 shadowPosition, float objectDepth, float shadowDepth
            , float alpha, float blurExtended, int index, bool pickable, float bumpDensity, float density,Vector2 textLocalPosition)
        {
            Position = position;
            ShadowPosition = shadowPosition;
            ObjectDepth = objectDepth;
            ShadowDepth = shadowDepth;
            Alpha = alpha;
            BlurExtended = blurExtended;
            Index = index;
            Pickable = pickable;
            BumpDensity = bumpDensity;
            Density = density;
            TextLocalPosition=textLocalPosition;
        }
    }
    public class Object
    {
       public InitializeInf ObjInf { get; set; }
       protected Body Body;
       public Vector2 RelToCenter;
       public bool Picked;
       public FarseerPhysics.Dynamics.Joints.FixedMouseJoint MouseJoint;
       public bool IsDisposed { get; set; }
       public bool IsPhysics { get; set; }
       public float Hit { get; set; }
       public float TotalHitBurden { get; set;}
       Vector3[] verticesPosition;   
       protected ShaderResourceView colorTextureRV, normalTextureRV;
       protected string ColorTextureFileName;
       protected string NormalTextureFileName;
       public bool IsDrawShadow { get; set; }
       public ObjectUIStatus Status { get; set; }
       public int GenerateRate;
       public bool Petrified;
       public bool WillBeRendered { get; set; }
       public SceneManager SceneMgr { get; set; }
       public void SetObjectDepth(float depth)
       {
           ObjInf.ObjectDepth = depth;
       }
       public Object()
       {
           WillBeRendered = false;
           GenerateRate = 1;
           Petrified = false;
       }
       public float GetObjectDepth()
       {
           return ObjInf.ObjectDepth;
       }
       public void SetIndex(int idx)
       {
           ObjInf.Index = idx;
       }
       public int GetIndex()
       {
           return ObjInf.Index;
       }
       public virtual void Render(Shader shader, Device device, Lighting lighting, Camera camera, float castBoardZ)
       {
           
           return;
       }
       public bool SetTextures(Device device, Shader shader ,string colorTextureFileName, string normalTextureFileName,TextureManager textureMgr)
        {

           
            if (colorTextureFileName != null)
            {
              
                colorTextureRV = new ShaderResourceView(device, textureMgr.MatchTextures(device,colorTextureFileName));
               // shader.GetEffect().GetVariableByName("colorTexture").AsResource().SetResource(colorTextureRV);
            }
            if (normalTextureFileName != null)
            {

                normalTextureRV = new ShaderResourceView(device, textureMgr.MatchTextures(device, normalTextureFileName));
               // shader.GetEffect().GetVariableByName("normalTexture").AsResource().SetResource(normalTextureRV);
            }
            
            return true;
        }
     
       public bool ChangeTextures(Device device, Shader shader,string colorTextureFileName, string normalTextureFileName,TextureManager textureMgr)
       {

        
               if (!colorTextureRV.Disposed)
               {
                   colorTextureRV.Dispose();
               }
               colorTextureRV = new ShaderResourceView(device, textureMgr.MatchTextures(device, colorTextureFileName));
               // shader.GetEffect().GetVariableByName("colorTexture").AsResource().SetResource(colorTextureRV);
      
               if (!normalTextureRV.Disposed)
               {
                   normalTextureRV.Dispose();
               }
               normalTextureRV = new ShaderResourceView(device, textureMgr.MatchTextures(device, normalTextureFileName));
               // shader.GetEffect().GetVariableByName("normalTexture").AsResource().SetResource(normalTextureRV);

           return true;
       }
        public void EnablePhysics()
        {
            if(IsPhysics)
            Body.Enabled = true;
        }
        public void DisablePhysics()
        {
            if (IsPhysics)
            Body.Enabled = false;
        }
       public Vector3 GetPosition()
       {
           if (IsPhysics)
           return new Vector3(Body.Position.X, Body.Position.Y, ObjInf.ObjectDepth);
           else
           return new Vector3(ObjInf.Position.X, ObjInf.Position.Y, ObjInf.ObjectDepth);
       }
       public Body GetBody()
       {
           return Body;
       }
       public void AddJoint(World world, Object jointed, bool enableLimit, float maxMotorTorque, float motorSpeed, bool enableMotor, Vec2 localA,Vec2 localB)
       {
           if (IsPhysics)
           {
               FarseerPhysics.Dynamics.Joints.RevoluteJoint joint = new FarseerPhysics.Dynamics.Joints.RevoluteJoint(Body, jointed.GetBody(), localA, localB);
               joint.LimitEnabled = enableLimit;
               joint.MaxMotorTorque = maxMotorTorque;
               joint.MotorSpeed = motorSpeed;
               joint.MotorEnabled = enableMotor;
               world.AddJoint(joint);
           }
       }
       public void SetLayer(Category category)
       {
           if (IsPhysics)
           {
                Body.CollisionCategories = category;
                Body.CollidesWith = category;
           }
          // Body.CollisionCategories = Category.Cat2;
       }
     
       public void initializeMouseJoint(World world)
       {
           if (IsPhysics)
           {
               //Initialize Mouse Joint
               MouseJoint = new FarseerPhysics.Dynamics.Joints.FixedMouseJoint(Body,
               new Vec2(Body.Position.X, Body.Position.Y));
               MouseJoint.MaxForce = 500000.0f;
               MouseJoint.Frequency = 100.1f;
               MouseJoint.Breakpoint = 50000.0f;
               MouseJoint.Enabled = false;
               world.AddJoint(MouseJoint);
           }

       }
       public void SetPickable(bool pickable)
       {
           ObjInf.Pickable = pickable;
       }
       protected void InitializeGeometry(Device device)
       {
           IsDisposed = false;
           Status = ObjectUIStatus.Released;
           float[] vertices = new[]
                                    {  
                                            -1.0f,  1.0f, -0.2f, 0.0f, 0.0f , 0.0f , 0.0f,-1.0f,1.0f,0.0f,0.0f,0.0f,-1.0f,0.0f,
                                             1.0f, 1.0f ,-0.2f,  1.0f, 0.0f , 0.0f  ,0.0f,-1.0f,1.0f,0.0f,0.0f,0.0f,-1.0f,0.0f,
                                            -1.0f, -1.0f ,-0.2f ,0.0f ,1.0f , 0.0f , 0.0f, -1.0f,1.0f,0.0f,0.0f,0.0f,-1.0f,0.0f,
                                            -1.0f ,-1.0f ,-0.2f ,0.0f, 1.0f , 0.0f , 0.0f ,-1.0f,1.0f,0.0f,0.0f,0.0f,-1.0f,0.0f,
                                             1.0f , 1.0f, -0.2f ,1.0f ,0.0f , 0.0f , 0.0f ,-1.0f,1.0f,0.0f,0.0f,0.0f,-1.0f,0.0f,
                                             1.0f,  -1.0f, -0.2f ,1.0f, 1.0f , 0.0f  ,0.0f ,-1.0f,1.0f,0.0f,0.0f,0.0f,-1.0f,0.0f
                                    };
                verticesPosition = new Vector3[]
                { 
                    new Vector3(-1.0f,  1.0f, -0.2f),
                    new Vector3(1.0f, 1.0f ,-0.2f),
                    new Vector3(-1.0f, -1.0f ,-0.2f),
                    new Vector3(-1.0f ,-1.0f ,-0.2f),
                    new Vector3(1.0f , 1.0f, -0.2f),
                    new Vector3(1.0f,  -1.0f, -0.2f)
                };

            short[] faces = new[]
                                {
                                        (short)0, (short)1, (short)2,
                                        (short)3, (short)4, (short)5
                                };

            // Creating vertex buffer
            const int vertexSize = sizeof(float) * 14;
            var stream = new DataStream(6 * vertexSize, true, true);
            stream.WriteRange(vertices);
            stream.Position = 0;
            
            var vertexBuffer = new SlimDX.Direct3D11.Buffer(device, stream, new BufferDescription
            {
                BindFlags = BindFlags.VertexBuffer,
                CpuAccessFlags = CpuAccessFlags.None,
                OptionFlags = ResourceOptionFlags.None,
                SizeInBytes = (int)stream.Length,
                Usage = ResourceUsage.Default
            });
            stream.Dispose();

            // Index buffer
            stream = new DataStream(6 * 2, true, true);
            stream.WriteRange(faces);
            stream.Position = 0;

            var indices = new SlimDX.Direct3D11.Buffer(device, stream, new BufferDescription
            {
                BindFlags = BindFlags.IndexBuffer,
                CpuAccessFlags = CpuAccessFlags.None,
                OptionFlags = ResourceOptionFlags.None,
                SizeInBytes = (int)stream.Length,
                Usage = ResourceUsage.Default
            });
            stream.Dispose();

            // Uploading to the device
            device.ImmediateContext.InputAssembler.PrimitiveTopology = PrimitiveTopology.TriangleList;
            device.ImmediateContext.InputAssembler.SetVertexBuffers(0, new VertexBufferBinding(vertexBuffer, vertexSize, 0));
            device.ImmediateContext.InputAssembler.SetIndexBuffer(indices, Format.R16_UInt, 0);
       }
       
        //To judge whether the object has been touched.
       public Vector2 Get2DPosition()
       {
           Camera camera = new Camera(); 
           if (SceneMgr != null)
           {
               camera=SceneMgr.GetCamera();
           }
           else
           {
               return new Vector2(0.0f, 0.0f);
           }

           Vector3 worldPosition=GetPosition();
           worldPosition.Z = 0.0f;
           Matrix worldViewProj = camera.GetViewMatrix() * camera.GetProjectionMatrix();
           float x = worldPosition.X * worldViewProj.get_Columns(0).X + worldPosition.Y * worldViewProj.get_Columns(0).Y + worldPosition.Z * worldViewProj.get_Columns(0).Z + worldViewProj.get_Columns(0).W;
           float y = worldPosition.X * worldViewProj.get_Columns(1).X + worldPosition.Y * worldViewProj.get_Columns(1).Y + worldPosition.Z * worldViewProj.get_Columns(1).Z + worldViewProj.get_Columns(1).W;
           float z = worldPosition.X * worldViewProj.get_Columns(2).X + worldPosition.Y * worldViewProj.get_Columns(2).Y + worldPosition.Z * worldViewProj.get_Columns(2).Z + worldViewProj.get_Columns(2).W;
           float w = worldPosition.X * worldViewProj.get_Columns(3).X + worldPosition.Y * worldViewProj.get_Columns(3).Y + worldPosition.Z * worldViewProj.get_Columns(3).Z + worldViewProj.get_Columns(3).W;
           Vector4 PosOnScreen = new Vector4(x, y, z, w);
           Vector2 screenCoord;
           screenCoord.X = PosOnScreen.X / PosOnScreen.W;
           screenCoord.Y = -PosOnScreen.Y / PosOnScreen.W;
           screenCoord.X = screenCoord.X / 2.0f + 0.5f;
           screenCoord.Y = screenCoord.Y / 2.0f + 0.5f;

           Vector2 wndPos;
           wndPos.X = screenCoord.X * camera.GetWidth();
           wndPos.Y = screenCoord.Y * camera.GetHeight();
           return new Vector2(wndPos.X, wndPos.Y);
       }
       public bool Pick(float x, float y, out Vector3 worldPosition, Camera camera)
        { 
            worldPosition = new Vector3();
            if (ObjInf.Pickable)
            {
               
                Quaternion qw = Quaternion.RotationAxis(new Vector3(0.0f, 0.0f, 1.0f), ObjInf.Angle);
                Quaternion scaleR = new Quaternion(0.0f, 0.0f, 0.0f, 0.0f);
                Matrix worldMatrix = Matrix.Transformation(new Vector3(0.0f, 0.0f, 0.0f), scaleR,
                new Vector3(ObjInf.Scale, 0.01f), new Vector3(0.0f, 0.0f, 0.0f), qw, new Vector3(ObjInf.Position.X, ObjInf.Position.Y, ObjInf.ObjectDepth));
                Vector3 vPickRayDir, vPickRayOrig;
                Vector3 v;
                v.X = (((2.0f * x) / camera.GetWidth()) - 1) / camera.GetProjectionMatrix().M11;
                v.Y = -(((2.0f * y) / camera.GetHeight()) - 1) / camera.GetProjectionMatrix().M22;
                v.Z = 1.0f;
                Matrix mWorldView = worldMatrix * camera.GetViewMatrix();
                Matrix m;
                Matrix.Invert(ref mWorldView, out m);
                vPickRayDir.X = v.X * m.M11 + v.Y * m.M21 + v.Z * m.M31;
                vPickRayDir.Y = v.X * m.M12 + v.Y * m.M22 + v.Z * m.M32;
                vPickRayDir.Z = v.X * m.M13 + v.Y * m.M23 + v.Z * m.M33;
                vPickRayOrig.X = m.M41;
                vPickRayOrig.Y = m.M42;
                vPickRayOrig.Z = m.M43;
                int idx = 0;
                for (int i = 0; i < 2; i++)
                {
                    Vector3 v0 = verticesPosition[idx];
                    idx++;
                    Vector3 v1 = verticesPosition[idx];
                    idx++;
                    Vector3 v2 = verticesPosition[idx];
                    idx++;
                    float t = 0.0f;
                    float u = 0.0f;
                    float vv = 0.0f;
                    if (IntersectTriangle(vPickRayOrig, vPickRayDir, v0, v1, v2, ref t, ref u, ref vv))
                    {
                        Vector3 position = vPickRayOrig + vPickRayDir * t;
                        float X = position.X * worldMatrix.get_Columns(0).X + position.Y * worldMatrix.get_Columns(0).Y + position.Z * worldMatrix.get_Columns(0).Z + worldMatrix.get_Columns(0).W;
                        float Y = position.X * worldMatrix.get_Columns(1).X + position.Y * worldMatrix.get_Columns(1).Y + position.Z * worldMatrix.get_Columns(1).Z + worldMatrix.get_Columns(1).W;
                        float Z = position.X * worldMatrix.get_Columns(2).X + position.Y * worldMatrix.get_Columns(2).Y + position.Z * worldMatrix.get_Columns(2).Z + worldMatrix.get_Columns(2).W;
                        worldPosition = new Vector3(X, Y, Z);
                        return true;
                    }
                }
                return false;
            }
            return false;
        }
        bool IntersectTriangle(Vector3 orig, Vector3 dir, Vector3 v0, Vector3 v1, Vector3 v2,
            ref float t, ref float u, ref float v)
        {
            Vector3 edge1 = v1 - v0;
            Vector3 edge2 = v2 - v0;
            // Begin calculating determinant - also used to calculate U parameter
            Vector3 pvec;
            Vector3.Cross(ref dir, ref edge2, out pvec);//(&pvec, &dir, &edge2);
            // If determinant is near zero, ray lies in plane of triangle
            float det = Vector3.Dot(edge1, pvec);// D3DXVec3Dot(&edge1, &pvec);
            Vector3 tvec;
            if (det > 0)
            {
                tvec = orig - v0;
            }
            else
            {
                tvec = v0 - orig;
                det = -det;
            }
            if (det < 0.0001f)
                return false;
            // Calculate U parameter and test bounds
            u = Vector3.Dot(tvec, pvec);// D3DXVec3Dot(&tvec, &pvec);
            if (u < 0.0f || u > det)
                return false;
            // Prepare to test V parameter
            Vector3 qvec;
            Vector3.Cross(ref tvec, ref edge1, out qvec);
            // Calculate V parameter and test bounds
            v = Vector3.Dot(dir, qvec); //D3DXVec3Dot(&dir, &qvec);
            if (v < 0.0f || u + v > det)
                return false;
            // Calculate t, scale parameters, ray intersects triangle
            t = Vector3.Dot(edge2, qvec); //D3DXVec3Dot(&edge2, &qvec);
            float fInvDet = 1.0f / det;
            t *= fInvDet;
            u *= fInvDet;
            v *= fInvDet;
            return true;
        }
        //Draw Text
        
      
        public void Dispose(World world)
        {
           // Body.Dispose();
            if (IsPhysics)
            {
                world.RemoveBody(Body);
                world.RemoveJoint(MouseJoint);
            }
            
            colorTextureRV.Dispose();
            normalTextureRV.Dispose();
            IsDisposed = true;
        }
        //The explosion effect will disable the physics and expand the rendering object
        //public void Explosion(Vec2 scale,float expandRate,float disappearRate)
        //{
        //    Body.Enabled = false;
        //    if (ObjInf.Scale.X < scale.X && ObjInf.Scale.Y < scale.Y)
        //    {
        //        ObjInf.Scale.X += expandRate;
        //        ObjInf.Scale.Y += expandRate;
        //    }
        //    else
        //    {
        //        ObjInf.Alpha -= disappearRate;
        //        ObjInf.ShadowDarkness -= disappearRate;
        //    }
        //}


    }
    

}
