﻿
using SlimDX;
using SlimDX.D3DCompiler;
using SlimDX.Direct3D11;
using SlimDX.DXGI;
using Device = SlimDX.Direct3D11.Device;
using Resource = SlimDX.Direct3D11.Resource;
enum ShapeType { CIRCLE, RECTANGLE, IRREGULAR ,CIRCLESHADOW,RECTANGLESHADOW,IRREGULARSHADOW};
namespace LittleBigTable
{
    using Vec2 = Microsoft.Xna.Framework.Vector2;
    public class Shader
    {
        ShaderResourceView colorTextureRV;
        ShaderResourceView normalTextureRV;
        Effect effect;
        InputLayout layout;
        
        ShapeType    type;
        public void CompileShader(Device device, string shaderFileName, string profile)
        {
            using (ShaderBytecode byteCode = ShaderBytecode.CompileFromFile(shaderFileName, profile, ShaderFlags.OptimizationLevel3, EffectFlags.None))
            {
                effect = new Effect(device, byteCode);
            }
            EffectTechnique technique;
            //initilize the pass to null
            EffectPass pass = null;

            effect.GetTechniqueByName("Render").GetPassByName("drawRectangle").Apply(device.ImmediateContext);
                    technique = effect.GetTechniqueByName("Render");
                    pass = technique.GetPassByName("drawRectangle");
                    layout = new InputLayout(device, pass.Description.Signature, new[] {
                                new InputElement("POSITION", 0, Format.R32G32B32_Float, 0, 0),
                                new InputElement("TEXCOORD", 0, Format.R32G32_Float, 12, 0),
                                new InputElement("NORMAL", 0, Format.R32G32B32_Float, 20, 0),
                                new InputElement("TANGENT", 0, Format.R32G32B32_Float, 32, 0),
                                new InputElement("BINORMAL", 0, Format.R32G32B32_Float, 44, 0)
                });
            device.ImmediateContext.InputAssembler.InputLayout = layout;
        }
        public Effect GetEffect()
        {
            return effect;
        }
        public InputLayout GetLayout()
        {
            return layout;
        }
        public bool SetTextures(Device device, string colorTextureFileName, string normalTextureFileName)
        {
            colorTextureRV = new ShaderResourceView(device, Texture2D.FromFile(device, colorTextureFileName));

                effect.GetVariableByName("colorTexture").AsResource().SetResource(colorTextureRV);

                normalTextureRV = new ShaderResourceView(device, Texture2D.FromFile(device, normalTextureFileName));

                effect.GetVariableByName("normalTexture").AsResource().SetResource(normalTextureRV);

            return true;
        }
        public void SetCircle(Device device, float radius, Vector3 position, float angle, float alpha, float bumpDensity)
        {
            type=ShapeType.CIRCLE;
            Vector2 scale=new Vector2(radius,radius);
            Quaternion scaleR= new Quaternion(0.0f,0.0f,0.0f,0.0f);
            Quaternion quaternion = Quaternion.RotationAxis(new Vector3(0.0f, 0.0f, 1.0f), angle);
            Matrix worldMatrix=Matrix.Transformation(new Vector3(0.0f, 0.0f, 0.0f), scaleR,
            new Vector3(scale, 0.01f), new Vector3(0.0f, 0.0f, 0.0f), quaternion, position);
            effect.GetVariableByName("scale").AsVector().Set(scale);
            effect.GetVariableByName("radius").AsScalar().Set(radius);
            effect.GetVariableByName("worldMatrix").AsMatrix().SetMatrix(worldMatrix);
            effect.GetVariableByName("centerPosition").AsVector().Set(new Vector4(position.X, position.Y, position.Z, 1.0f));
            effect.GetVariableByName("alpha").AsScalar().Set(alpha);
            effect.GetVariableByName("bumpDensity").AsScalar().Set(bumpDensity);
            effect.GetTechniqueByName("Render").GetPassByName("drawCircle").Apply(device.ImmediateContext);
           
        }
        public void SetRectangle(Device device, float width, float height, Vector3 position, float angle, float alpha, float bumpDensity)
        {
            type = ShapeType.RECTANGLE;
            Vector2 scale=new Vector2(width,height);
            Quaternion scaleR= new Quaternion(0.0f,0.0f,0.0f,0.0f);
            Quaternion quaternion = Quaternion.RotationAxis(new Vector3(0.0f, 0.0f, 1.0f), angle);
            Matrix worldMatrix=Matrix.Transformation(new Vector3(0.0f, 0.0f, 0.0f), scaleR,
            new Vector3(scale, 0.01f), new Vector3(0.0f, 0.0f, 0.0f), quaternion, position);
            effect.GetVariableByName("scale").AsVector().Set(scale);
            effect.GetVariableByName("worldMatrix").AsMatrix().SetMatrix(worldMatrix);
            effect.GetVariableByName("alpha").AsScalar().Set(alpha);
            effect.GetVariableByName("bumpDensity").AsScalar().Set(bumpDensity);
            effect.GetTechniqueByName("Render").GetPassByName("drawRectangle").Apply(device.ImmediateContext);
        }
        public void Set3DModel(Device device, Vector3 position,Vector3 scale, Quaternion quaternion, float alpha)
        {
            Quaternion scaleR= new Quaternion(0.0f,0.0f,0.0f,0.0f);
            // = Quaternion.RotationAxis(Axis, angle);
            
            Matrix worldMatrix=Matrix.Transformation(new Vector3(0.0f, 0.0f, 0.0f), scaleR,
            scale, new Vector3(0.0f, 0.0f, 0.0f), quaternion, position);
            effect.GetVariableByName("scale").AsVector().Set(scale);
            effect.GetVariableByName("worldMatrix").AsMatrix().SetMatrix(worldMatrix);
            effect.GetVariableByName("alpha").AsScalar().Set(alpha);
            effect.GetVariableByName("bumpDensity").AsScalar().Set(0.1f);
            effect.GetTechniqueByName("Render").GetPassByName("drawRectangle").Apply(device.ImmediateContext);

        }
        public void SetIrregular(Device device,float width,float height,Vector3 position, float angle,float alpha,float bumpDensity)
        {
            type = ShapeType.IRREGULAR;
            Vector2 scale=new Vector2(width,height);
            Quaternion scaleR= new Quaternion(0.0f,0.0f,0.0f,0.0f);
            Quaternion quaternion = Quaternion.RotationAxis(new Vector3(0.0f, 0.0f, 1.0f), angle);
            Matrix worldMatrix=Matrix.Transformation(new Vector3(0.0f, 0.0f, 0.0f), scaleR,
            new Vector3(scale, 0.01f), new Vector3(0.0f, 0.0f, 0.0f), quaternion, position);
            effect.GetVariableByName("scale").AsVector().Set(scale);
            effect.GetVariableByName("worldMatrix").AsMatrix().SetMatrix(worldMatrix);
            effect.GetVariableByName("alpha").AsScalar().Set(alpha);
            effect.GetVariableByName("bumpDensity").AsScalar().Set(bumpDensity);
            effect.GetTechniqueByName("Render").GetPassByName("drawIrregularShape").Apply(device.ImmediateContext);
           
       
        }
        public void SetIrregularScreen(Device device, float width, float height, Vector3 position, float angle, float alpha, float bumpDensity)
        {
            Vector2 scale = new Vector2(width, height);
            Quaternion scaleR = new Quaternion(0.0f, 0.0f, 0.0f, 0.0f);
            Quaternion quaternion = Quaternion.RotationAxis(new Vector3(0.0f, 0.0f, 1.0f), angle);
            Matrix worldMatrix = Matrix.Transformation(new Vector3(0.0f, 0.0f, 0.0f), scaleR,
            new Vector3(scale, 0.01f), new Vector3(0.0f, 0.0f, 0.0f), quaternion, position);
            effect.GetVariableByName("scale").AsVector().Set(scale);
            effect.GetVariableByName("worldMatrix").AsMatrix().SetMatrix(worldMatrix);
            effect.GetVariableByName("alpha").AsScalar().Set(alpha);
            effect.GetVariableByName("bumpDensity").AsScalar().Set(bumpDensity);
            effect.GetTechniqueByName("Render").GetPassByName("drawIrregularShapeScreen").Apply(device.ImmediateContext);
        }
        public void SetCircleShadow(Device device, float radius, Vector3 position, float angle,
             float blurExtended, float shadowDarkness)
        {
            type = ShapeType.CIRCLESHADOW;
            Vector2 scale = new Vector2(radius, radius);
            Quaternion scaleR = new Quaternion(0.0f, 0.0f, 0.0f, 0.0f);
            Quaternion quaternion = Quaternion.RotationAxis(new Vector3(0.0f, 0.0f, 1.0f), angle);
            Matrix worldMatrix = Matrix.Transformation(new Vector3(0.0f, 0.0f, 0.0f), scaleR,
            new Vector3(scale, 0.01f), new Vector3(0.0f, 0.0f, 0.0f), quaternion, position);
            effect.GetVariableByName("scale").AsVector().Set(scale);
            effect.GetVariableByName("radius").AsScalar().Set(radius);
            effect.GetVariableByName("worldMatrix").AsMatrix().SetMatrix(worldMatrix);
            effect.GetVariableByName("centerPosition").AsVector().Set(new Vector4(position.X, position.Y, position.Z, 1.0f));
            effect.GetVariableByName("blurExtended").AsScalar().Set(blurExtended);
            effect.GetVariableByName("shadowDarkness").AsScalar().Set(shadowDarkness);
            effect.GetTechniqueByName("RenderShadow").GetPassByName("drawCircleShadow").Apply(device.ImmediateContext);
                   
            
        }
        public void SetRectangleShadow(Device device,  float width, float height, 
            Vector3 position, float angle,float blurExtended, float shadowDarkness)
        {
            type = ShapeType.RECTANGLE;
            Vector2 scale = new Vector2(width, height);
            Quaternion scaleR = new Quaternion(0.0f, 0.0f, 0.0f, 0.0f);
            Quaternion quaternion = Quaternion.RotationAxis(new Vector3(0.0f, 0.0f, 1.0f), angle);
            Matrix worldMatrix = Matrix.Transformation(new Vector3(0.0f, 0.0f, 0.0f), scaleR,
            new Vector3(scale, 0.01f), new Vector3(0.0f, 0.0f, 0.0f), quaternion, position);
            effect.GetVariableByName("scale").AsVector().Set(scale);
            effect.GetVariableByName("worldMatrix").AsMatrix().SetMatrix(worldMatrix);
            effect.GetVariableByName("blurExtended").AsScalar().Set(blurExtended);
            effect.GetVariableByName("shadowDarkness").AsScalar().Set(shadowDarkness);
            effect.GetTechniqueByName("RenderShadow").GetPassByName("drawRectangleShadow").Apply(device.ImmediateContext);
                
        }
        public void SetIrregularShadow(Device device,  float width, float height, 
            Vector3 position, float angle, float blurExtended, float shadowDarkness)
        {
            type = ShapeType.IRREGULAR;
            Vector2 scale = new Vector2(width, height);
            Quaternion scaleR = new Quaternion(0.0f, 0.0f, 0.0f, 0.0f);
            Quaternion quaternion = Quaternion.RotationAxis(new Vector3(0.0f, 0.0f, 1.0f), angle);
            Matrix worldMatrix = Matrix.Transformation(new Vector3(0.0f, 0.0f, 0.0f), scaleR,
            new Vector3(scale, 0.01f), new Vector3(0.0f, 0.0f, 0.0f), quaternion, position);
            effect.GetVariableByName("scale").AsVector().Set(scale);
            effect.GetVariableByName("worldMatrix").AsMatrix().SetMatrix(worldMatrix);
            effect.GetVariableByName("blurExtended").AsScalar().Set(blurExtended);
            effect.GetVariableByName("shadowDarkness").AsScalar().Set(shadowDarkness);
            effect.GetTechniqueByName("RenderShadow").GetPassByName("drawIrregularShadow").Apply(device.ImmediateContext);         
        }
        public void InitializeInputLayout()
        {

        }
        public void Render(Device device, Lighting lighting,Camera camera ,int verticeNum)
        {
            Matrix viewMatrix;

            viewMatrix = Matrix.LookAtLH(camera.GetPosition(), camera.GetLookAt(), camera.GetUpAxis());
            const float fov = 0.8f;
            Matrix projectionMatrix = Matrix.PerspectiveFovLH(fov, camera.GetWidth() / camera.GetHeight(), 0.1f, 1000.0f);
            effect.GetVariableByName("viewMatrix").AsMatrix().SetMatrix(camera.GetViewMatrix());
            effect.GetVariableByName("projectionMatrix").AsMatrix().SetMatrix(camera.GetProjectionMatrix());
            effect.GetVariableByName("ambiValue").AsScalar().Set(lighting.GetAmbientValue());
            effect.GetVariableByName("spotIntensy").AsScalar().Set(lighting.GetSpotIntensy());
            effect.GetVariableByName("lightPosition").AsVector().Set(new Vector4(lighting.GetLightPosition(),1.0f));
            Matrix ViewMatrix;
            Vector3 lPosition = new Vector3();
            lPosition = lighting.GetLightPosition();
            Vector3 LookAt = new Vector3(0.0f,0.0f,0.0f);
            Vector3 Up = new Vector3(0.0f,1.0f,0.0f);
            Matrix.LookAtLH(ref lPosition, ref LookAt, ref Up, out ViewMatrix);
            Matrix lightViewProj = ViewMatrix * camera.GetProjectionMatrix();
            effect.GetVariableByName("lightViewProj").AsMatrix().SetMatrix(lightViewProj);

            device.ImmediateContext.DrawIndexed(verticeNum, 0, 0);
        }
    }
}
