RasterizerState backFaceCulling
{
	cullmode = back;
};
SamplerState MeshTextureSampler
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = MIRROR;
    AddressV = MIRROR;	
};
SamplerState g_samLinear
{
	Filter = MIN_MAG_MIP_LINEAR;
	AddressU = CLAMP;
	AddressV = CLAMP;
};
cbuffer Matrix
{
	matrix worldMatrix;
	matrix viewMatrix;
	matrix projectionMatrix;
}
cbuffer Objects
{
	float4 centerPosition;
	float2 scale;
	float  alpha;
	float  radius;
	float  bumpDensity;
	float  blurExtended;
	float  shadowDarkness;
}
BlendState SrcAlphaBlendingAdd
{
	BlendEnable[0] = TRUE;
	
	SrcBlend = SRC_ALPHA;
	DestBlend = INV_SRC_ALPHA;

	BlendOp = ADD;
	SrcBlendAlpha = One;
	DestBlendAlpha = One;

	BlendOpAlpha = ADD;
	RenderTargetWriteMask[0] = 0x0F;
};

cbuffer Camera 
{
	float4 cameraPosition;
}
cbuffer Light
{
	float  type;
	float4 lightPosition;
	float3 lightDirection;
	float  ambiValue;
	float  spotIntensy;
	matrix lightViewProj;
}
struct VS_IN
{
	float4 pos : POSITION;
	float2 tex : TEXCOORD0;
	float3 normal : NORMAL;
	float3 tangent : TANGENT;
	float3 binormal : BINORMAL;
};

struct PS_IN
{
	float4 pos : SV_POSITION;
	float4 wpos : WPOS;
	float4 lpos : TEXCOORD1;		//vertex with regard to light view

	float2 tex : TEXCOORD0;
	float3 normal : NORMAL;
	float3 tangent : TANGENT;
	float3 binormal : BINORMAL;	
	float4 worldPosition : WORLD;
};

// Vertex Shader
PS_IN vertexShader( VS_IN input )
{

	PS_IN output = (PS_IN)0;	
	output.wpos=input.pos;
	output.lpos=mul( input.pos, mul(worldMatrix,lightViewProj));

	// Change the position vector to be 4 units for proper matrix calculations.
    input.pos.w = 1.0f;
    output.pos = mul(input.pos, worldMatrix);
    output.pos = mul(output.pos, viewMatrix);
    output.pos = mul(output.pos, projectionMatrix);

	// Store the texture coordinates for the pixel shader.
	output.tex = input.tex;
    
    // Calculate the normal vector against the world matrix only and then normalize the final value.
    output.normal = mul(input.normal, (float3x3)worldMatrix);
    output.normal = normalize(output.normal);
   

	// Calculate the tangent vector against the world matrix only and then normalize the final value.
    output.tangent = mul(input.tangent, (float3x3)worldMatrix);
    output.tangent = normalize(output.tangent);
   

    // Calculate the binormal vector against the world matrix only and then normalize the final value.
    output.binormal = mul(input.binormal, (float3x3)worldMatrix);
    output.binormal = normalize(output.binormal);
   

	// Calculate the position of the vertex in the world.
    output.worldPosition = mul(input.pos, worldMatrix);
	
	return output;
}

PS_IN vertexShadowShader(VS_IN input)
{
	PS_IN output = (PS_IN)0;
	input.pos.w = 1.0f;
    output.pos = mul(input.pos, worldMatrix);
    output.pos = mul(output.pos, viewMatrix);
    output.pos = mul(output.pos, projectionMatrix);
	output.tex=input.tex;
	output.worldPosition = mul(input.pos, worldMatrix);
	return output;
}
PS_IN vertexScreenShader(VS_IN input)
{
	
}

Texture2D colorTexture;
Texture2D normalTexture;
Texture2D g_textOverlay;
Texture2D shadowMap;
struct TextVS_IN
{
	float4 pos : POSITION;
	float2 tex : TEXCOORD0;
};
 
struct TextPS_IN
{
	float4 pos : SV_POSITION;
	float2 tex : TEXCOORD0;
};
 
TextPS_IN TextVS( TextVS_IN input )
{
	TextPS_IN output = (TextPS_IN)0;
	output.pos = input.pos;
	output.tex = input.tex;
	return output;
}
 
float4 TextPS( TextPS_IN input ) : SV_Target
{
	float4 color =  g_textOverlay.Sample(g_samLinear, input.tex);
	if(color.x==0&&color.y==0&&color.z==0&&color.w==0)
	{
		color=float4(1.0f,0.0f,0.0f,1.0f);
	}
	
	return float4(1.0f,0.0f,0.0f,1.0f);
}


float4 drawBumpSurface(PS_IN input)
{
	float4 textureColor;
    float4 bumpMap;
    float3 bumpNormal;
    float lightIntensity;
    float4 color;
	float4 diffuseColor=float4(1.0f,1.0f,1.0f,1.0f);
	float4 LightDirection = float4 (1.0f,1.0f,1.0f,0.0f); 
	LightDirection = normalize(LightDirection);
	// Sample the texture pixel at this location.

    textureColor = colorTexture.Sample(MeshTextureSampler, input.tex);
	//Transform the coordinator
   // float2 transformedPosition=float2((input.tex.x-0.5)*(scale.x/bumpDensity)+0.5f,(input.tex.y-0.5)*(scale.y/bumpDensity)+0.5f);

    bumpMap = normalTexture.Sample(MeshTextureSampler, input.tex);
    bumpMap = (bumpMap * 2.0f) - 1.0f;
    // Calculate the normal from the data in the bump map.
    bumpNormal = input.normal + bumpMap.x * input.tangent + bumpMap.y * input.binormal;
    bumpNormal = normalize(bumpNormal);

    lightIntensity =saturate(dot(bumpNormal, LightDirection.xyz));

    

	color = saturate(diffuseColor * lightIntensity);
        color=color*lightIntensity* textureColor*spotIntensy+float4(1.0f,1.0f,1.0f,1.0f)*ambiValue;
	return color;
}
 
// Pixel Shader
//float4 circleBumpPixelShader( PS_IN input ) : SV_Target
//{
//	float4 color;
//	float4 colorReal;
//	color=drawBumpSurface(input);

//    float toCenter= length(float2(input.tex.x-0.5,input.tex.y-0.5));
//	if(toCenter<0.5f)
//	{
//		colorReal=color;
//		colorReal.w=alpha;
//		if(toCenter>(0.5-0.04/radius))
//		{
//			colorReal.w=alpha-0.2;
//		}
//	}
//	else
//	{
//		discard;
//	}
//	return colorReal;
//}
float4 circleBumpPixelShader( PS_IN input ) : SV_Target
{
	float4 color;
	float4 colorReal;
	color=drawBumpSurface(input);
	float3 distance=input.worldPosition.xyz-centerPosition.xyz;
	float diatance= length(distance);
	if(diatance<=radius)
	{
		colorReal=color;
		colorReal.w=alpha;
		if(diatance>=radius-0.1)
		{	
			float toEdge=radius-diatance;
			float a=alpha*toEdge*10;
			colorReal.w=a;
		}
	}
	else
	{
		discard;
	}
	return colorReal;
}

float4 rectangleBumpPixelShader( PS_IN input ) : SV_Target
{
	float4 colorReal;
	colorReal= drawBumpSurface(input);	
	colorReal.w=alpha.x;
	return colorReal;
}

float4 irregularShapeBumpPixelShader( PS_IN input ) : SV_Target
{	
	float4 colorReal;
	float4 textureColor;
	colorReal=drawBumpSurface(input);	
	colorReal.w=alpha;
	textureColor=colorTexture.Sample(MeshTextureSampler, input.tex);
	if(textureColor.w<0.8f)
	{
		discard;
	}
	return colorReal;
}
float4 irregularShapeBumpScreenPS( PS_IN input ) : SV_Target
{	
	float4 colorReal;
	float4 textureColor;
	colorReal=drawBumpSurface(input);	
	colorReal.w=alpha;
	textureColor=colorTexture.Sample(MeshTextureSampler, input.tex);
	if(textureColor.w<0.8f)
	{
		discard;
	}
	return colorReal;
}

float4 circleShadow( PS_IN input ) : SV_Target
{
	float3 distance=input.worldPosition.xyz-centerPosition.xyz;
	float4 colorReal;
	float transparency=shadowDarkness;
	
	if(length(distance)<radius)
	{
		if(length(distance)>=radius-blurExtended)
		{	
			float toEdge=radius-length(distance);
			float a=transparency*toEdge/blurExtended;
			return float4(0.0f,0.0f,0.0f,a);
		}
		return float4(0.0f,0.0f,0.0f,transparency);
	}
	else
	{	
		discard;
	}
	return colorReal;
}

float4 rectangleShadow( PS_IN input ) : SV_Target
{
	float4 color = float4(0.0f, 0.0f, 0.0f, shadowDarkness);
	float  marginX=blurExtended/(scale.x*2);
	float  marginY=blurExtended/(scale.y*2);
	float  darknessX= shadowDarkness/marginX;
	float  darknessY= shadowDarkness/marginY;

	float  x=0.5-abs(0.5-input.tex.x);
	float  y=0.5-abs(0.5-input.tex.y);
	color.w=shadowDarkness;

	// Add the nine horizontal pixels to the color by the specific weight of each.

	if(input.tex.x>(1-marginX)&&input.tex.x*scale.x>input.tex.y*scale.y+scale.x-scale.y&&input.tex.y*scale.y>(scale.x-input.tex.x*scale.x))
	color.w=(1.0f-input.tex.x)*darknessX;

	if(input.tex.y>(1-marginY)&&input.tex.y*scale.y>(scale.y-input.tex.x*scale.x)&&input.tex.x*scale.x<input.tex.y*scale.y+scale.x-scale.y)
	color.w=(1.0f-input.tex.y)*darknessY;

	if(input.tex.y<marginY&&input.tex.y*scale.y<(scale.x-input.tex.x*scale.x)&&input.tex.y*scale.y<input.tex.x*scale.x)
	color.w=input.tex.y*darknessY;

	if(input.tex.x<marginX&&input.tex.y*scale.y<(scale.y-input.tex.x*scale.x)&&input.tex.x*scale.x<input.tex.y*scale.y)
	color.w=input.tex.x*darknessX;
	

	

	return color;
}

float4 irregularShadow( PS_IN input ) : SV_Target
{
	// Initialize the color to black.
	float4 color;
	color=float4(0.0f,0.0f,0.0f,0.0f);
	//increase the numPixel will lower the efficiency
	int numPixel=5;
	float2 position;
	float2 transformedPosition;
	
	float driftX=(blurExtended/numPixel)/(scale.x*2);
	float driftY=(blurExtended/numPixel)/(scale.y*2);
	int fast=1;	
	if(color.w<0.0)
	{
	  discard;
	}
	for(int i=-numPixel/2;i<numPixel/2;i++)
	{
	   for(int j=-numPixel/2;j<numPixel/2;j++)
	   {
			transformedPosition=float2((input.tex.x-0.5f)*1.1f,(input.tex.y-0.5f)*1.1f)+float2(0.5f,0.5f);
			position= transformedPosition+float2(j*driftX,i*driftY);
			if(position.x>0.0f&&position.y>0.0f&&position.x<1.0f&&position.y<1.0f)
			{
				color.w +=colorTexture.Sample(MeshTextureSampler, position).w;	
			 }
			fast+=1;
	    }
	}
	color.w=color.w/fast;
	color.w*=shadowDarkness;
	return color;
}

//Draw the shadow
PS_IN ShadowMapVS( VS_IN input )
{
	PS_IN output;
    output.pos = mul(input.pos, mul(worldMatrix,lightViewProj));		
    return output;  
}
void ShadowMapPS( PS_IN input ) {}

// Technique
technique10 ShadowMapRenderPassFrontFaces
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, ShadowMapVS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, ShadowMapPS() ) );
		//SetRasterizerState( backFaceCulling );		
    }
}
technique10 Render
{
	pass drawCircle
	{
		SetGeometryShader( 0 );
		SetVertexShader( CompileShader( vs_4_0, vertexShader() ) );
		SetPixelShader( CompileShader( ps_4_0, circleBumpPixelShader() ) );
		SetBlendState( SrcAlphaBlendingAdd, float4( 1.0f, 1.0f, 1.0f, 1.0f ), 0xFFFFFFFF );
	}
	pass drawRectangle
	{
		SetGeometryShader( 0 );
		SetVertexShader( CompileShader( vs_4_0, vertexShader() ) );
		SetPixelShader( CompileShader( ps_4_0, rectangleBumpPixelShader() ) );
	}
	pass drawIrregularShape
	{
		SetGeometryShader( 0 );
		SetVertexShader( CompileShader( vs_4_0, vertexShader() ) );
		SetPixelShader( CompileShader( ps_4_0, irregularShapeBumpPixelShader() ) );
		SetBlendState( SrcAlphaBlendingAdd, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
	}
	pass drawIrregularShapeScreen
	{
		SetGeometryShader( 0 );
		SetVertexShader( CompileShader( vs_4_0, vertexShader() ) );
		SetPixelShader( CompileShader( ps_4_0, irregularShapeBumpScreenPS() ) );
		SetBlendState( SrcAlphaBlendingAdd, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
	}
}

technique10 RenderShadow
{
	pass drawCircleShadow
	{
		SetGeometryShader( 0 );
		SetVertexShader( CompileShader( vs_4_0, vertexShadowShader() ) );
		SetPixelShader( CompileShader( ps_4_0, circleShadow() ) );
		//SetBlendState( SrcAlphaBlendingAdd, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
	}
	pass drawRectangleShadow
	{
		SetGeometryShader( 0 );
		SetVertexShader( CompileShader( vs_4_0, vertexShadowShader() ) );
		SetPixelShader( CompileShader( ps_4_0, rectangleShadow() ) );
		SetBlendState( SrcAlphaBlendingAdd, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
	}
	pass drawIrregularShadow
	{
		SetGeometryShader( 0 );
		SetVertexShader( CompileShader( vs_4_0, vertexShadowShader() ) );
		SetPixelShader( CompileShader( ps_4_0, irregularShadow() ) );
		SetBlendState( SrcAlphaBlendingAdd, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
	}
}
technique10 Text
{
	pass P0
	{
		SetGeometryShader( 0 );
		SetVertexShader( CompileShader( vs_4_0, TextVS() ) );
		SetPixelShader( CompileShader( ps_4_0, TextPS() ) );
		SetBlendState( SrcAlphaBlendingAdd, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
	}
}